# -*- coding: iso_8859_1 -*-

from aspylib import astro, photometry
import numpy as np
import matplotlib.pyplot as plt
import os

#------------------------------------------------------------------------------------
#	Purpose 
#------------------------------------------------------------------------------------
#
#	Calculates the instrumental magnitudes of asteroids and variable stars over a series of images.
#	The script must be placed in the folder that contains the images, and is started with a double click
#
#------------------------------------------------------------------------------------
#	Input parameters (basic)
#------------------------------------------------------------------------------------

#--- calibrated images ---
Folder = os.getcwd()
FileList = os.listdir(Folder)
Folder += "\\"
Imagelist = [Folder+f for f in FileList if (f[-5:]==".fits")]

#radii for aperture photometry (pixels)
#sizes = [[2,8,40],[2.5,8,40],[3,8,40],[3.5,8,40],[4,8,40],[4.5,8,40],[5,8,40],[5.5,8,40],[6,8,40]]
#sizes = [[4,8,40],[4.5,8,40],[5,8,40],[5.5,8,40],[6,8,40],[6.5,10,45],[7,10,45],[7.5,12,45],[8,12,45],[8.5,14,50],[9,14,50]]
#sizes = [[8,16,45],[8.5,16,50],[9,16,50],[9.5,16,55],[10,18,50],[10.5,18,50],[11,18,50],[11.5,18,50],[12,20,50]]
sizes = [[4,8,40],[4.5,8,40],[5,8,40],[5.5,8,40],[6,8,40],[6.5,10,45],[7,10,45],[7.5,12,45],[8,12,45],[8.5,14,50],[9,14,50]]

#--- names of output files ---
generic_name = Folder + 'target_'

#--- flag to shortcut image shift calculations ---
flag_calc = True	#if True, checks images and calculates shifts. If False, read results from file

#------------------------------------------------------------------------------------
#	Input parameters (advanced)
#------------------------------------------------------------------------------------

#--- semi-size of square areas used for star max detection and star fitting ---
fit_area = 8			#typically a few times (2-3) the star FWHM.
max_detection_area = 2		#max_detection_area = 2 is recommended. Can be SLIGHTLY increased for very large stars (FWHM >5 pixels).

#--- photometry method ---
#method = 'select'
#method = 'maximum'
method = 'centroid'

#--- names of output files ---
outputname1 = generic_name + "01_check_images.txt"
outputname2 = generic_name + "02_image_shifts.txt"
outputname3 = generic_name + "03_selected_stars.pdf"
outputname4 = generic_name + "04_inst_magnitudes.txt"

#####################################################################################
#------------------------------------------------------------------------------------
#	Script
#------------------------------------------------------------------------------------
print("")
print('*********************************************************************')
print('      SCRIPT FOR APERTURE PHOTOMETRY')
print('*********************************************************************')

#------ date information ------
dateobs = astro.julian_day(astro.get_headervalues(Imagelist, 'DATE-OBS'))
indices = np.argsort(dateobs)
Imagelist = [Imagelist[i] for i in indices]

#------------------------------------------------------------------------------------
#	selects images
#------------------------------------------------------------------------------------

if flag_calc:
	#------ checking quality of measured frames ------
	print("")
	print('---------------------------------------------------------------------')
	print('      Step 1 : checking the input images')
	print('---------------------------------------------------------------------')
	print("")
	Imagelist2 = photometry.check_pictures_quality(outputname1, Imagelist)

	#------ calculating shifts ------
	print("")
	print('---------------------------------------------------------------------')
	print('      Step 2 : calculation of image shifts')
	print('---------------------------------------------------------------------')
	print("")
	Imagelist3, shifts = photometry.calculate_shifts(outputname2, Imagelist2, fit_area = fit_area, nb_stars = 12)
else:
	Imagelist3, shifts = photometry.load_shifts(outputname2)
	ind = list(np.array(np.arange(1,len(Imagelist3)+1), dtype=np.int))
	plt.figure()
	plt.plot(ind,shifts[:,0], 'b')
	plt.plot(ind,shifts[:,1], 'g')
	plt.grid()
	plt.title('Image shifts: Vertical (blue), Horizontal (green)')
	plt.xlabel('Image number')
	plt.ylabel('Image shifts (pixels)')
	plt.show()

dateobs = astro.julian_day(astro.get_headervalues(Imagelist3, 'DATE-OBS'))
	
#------------------------------------------------------------------------------------
#	calculates instrumental magnitudes
#------------------------------------------------------------------------------------

#------ select stars ------
print("")
print('---------------------------------------------------------------------')
print('      Step 3 : selecting targets for photometry')
print('---------------------------------------------------------------------')
print("")
if method=='select':
	names, xy = photometry.select_stars(Imagelist3, shifts, outputname3, fit_area = 0, method = method)
elif method=='maximum':
	names, xy = photometry.select_stars(Imagelist3, shifts, outputname3, fit_area = max_detection_area, method = method)
elif method=='centroid':
	names, xy = photometry.select_stars(Imagelist3, shifts, outputname3, fit_area = fit_area, method = method)

#------ perform aperture photometry ------
print("")
print('---------------------------------------------------------------------')
print('      Step 4 : calculating aperture photometry')
print('---------------------------------------------------------------------')
print("")
mag = photometry.make_photometry(Imagelist3, sizes, xy, method=method)
photometry.write_magnitudes(outputname4, Imagelist3, names, xy, dateobs, sizes, mag)	

#now magnitudes can be accessed with mag[image, object, disk]
