# -*- coding: iso_8859_1 -*-

from aspylib import cdr, astro, conf
import numpy as np
import matplotlib.pyplot as plt

#------------------------------------------------------------------------------------
#	Purpose
#------------------------------------------------------------------------------------
#
#	Reads a list of lightcurve files (with CourbRot format) and makes a Fourier fit.
#	The following are calculated:
#	- the lightcurve period
#	- the min-to-max amplitude
#	- date of min and max magnitudes
#	- a phase ephemeris
#	The time delays due to light propagation are corrected at the start of the script execution.
#	Then, for stars all the dates used for the fit are heliocentric, for asteroids the dates are corrected for light propagation from asteroid to Earth.
#
#------------------------------------------------------------------------------------
#	Input parameters
#------------------------------------------------------------------------------------

#--- input files ---
Folder = conf.folder + "Mesures\\"

Filelist = ["File_1.txt", \
			"File_2.txt", \
			"File_3.txt", \
			"File_4.txt"]
Filelist = [Folder + ff for ff in Filelist]

#type of target (either "asteroid" or "star")
target = "asteroid"		
#target = "star"

#for time correction of variable star
alpha = astro.hours_to_decdegrees('10:45:42.19')
delta = astro.degrees_to_decdegrees('+27:57:54.2')

#for time correction of asteroid
file_DistanceToEarth = Folder + "dist.txt"

#for plotting the lightcurve
title = "1069 Planckia\nLightcurve with Fourier fit"

#False if the same reference stars are used for all the series (less fit parameters)
fit_offsets = True

#fit parameters
T0 = 2455690.0			#reference time (this parameter defines the time when phase = 0. It is also needed in the calculation to avoid too large numbers for the epochs)
period = 0.36		#starting period for the optimisation loop
nb_harmo = 4			#number of Fourier harmonics for fitting the lightcurve

#for phase ephemeris
eph_start = 2455822.5
eph_end = 2455842.5
eph_nbpts = 241

#------------------------------------------------------------------------------------
#	Script
#------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------
#	loading data
#------------------------------------------------------------------------------------
if fit_offsets:
	alldata = cdr.load_magdata(Filelist, zero_mean=True)
else:
	alldata = cdr.load_magdata(Filelist, zero_mean=False)

#------ time correction (choose asteroid or variable star) ------
if target == "asteroid":
	alldata = cdr.time_correction(alldata, 1, file_DistanceToEarth)			#time correction for asteroid
else:
	alldata = cdr.time_correction(alldata, 2, [alpha,delta])				#time correction for variable star

cdr.plot_magdata(alldata, 'Measurements')

#------------------------------------------------------------------------------------
#	fitting data
#------------------------------------------------------------------------------------

popt, pcov, period, dperiod, amplitude, maxdate, mindate = cdr.fit_CDR_fitperiod(alldata, period, T0, nb_harmo = nb_harmo, show_results = True, fit_offsets = fit_offsets)

#plotting lightcurve
x = np.linspace(0.,1.,10000)
y = 0.
plt.figure()

if fit_offsets:
	for j in range(nb_harmo):
		y = y + popt[j+1]*np.cos(np.double(j+1)*x*2.*np.pi) + popt[j+1+nb_harmo]*np.sin(np.double(j+1)*x*2.*np.pi)
	for i,dd in enumerate(alldata):
		plt.plot((dd[:,0]-T0)/popt[0]-np.floor((dd[:,0]-T0)/popt[0]), dd[:,1]-popt[i+1+2*nb_harmo],'+',c=conf.col[i%16])
		plt.plot((dd[:,0]-T0)/popt[0]-np.floor((dd[:,0]-T0)/popt[0])+1, dd[:,1]-popt[i+1+2*nb_harmo],'+',c=conf.col[i%16])
	plt.ylim([0.4,-0.4])
else:
	for j in range(nb_harmo):
		y = y + popt[j+1]*np.cos(np.double(j+1)*x*2.*np.pi) + popt[j+1+nb_harmo]*np.sin(np.double(j+1)*x*2.*np.pi)
	y += popt[1+2*nb_harmo]
	for i,dd in enumerate(alldata):
		plt.plot((dd[:,0]-T0)/popt[0]-np.floor((dd[:,0]-T0)/popt[0]), dd[:,1],'+',c=conf.col[i%16])
		plt.plot((dd[:,0]-T0)/popt[0]-np.floor((dd[:,0]-T0)/popt[0])+1, dd[:,1],'+',c=conf.col[i%16])
	plt.ylim([popt[1+2*nb_harmo]+1.2,popt[1+2*nb_harmo]-1.2])

plt.plot(x,y,'b')
plt.plot(x+1.,y,'b')
plt.xlim([0.,1.301])
plt.grid()
plt.xlabel('Phase')
plt.ylabel('Magnitude')
#ax = plt.gca()
#ax.set_ylim(ax.get_ylim()[::-1])
plt.title(title)
plt.show()

#------------------------------------------------------------------------------------
#	generating phase ephemerid
#------------------------------------------------------------------------------------

JJ = np.linspace(eph_start, eph_end, eph_nbpts)
print("")
print("------ phase ephemerids ------")
phase = (JJ-T0)/popt[0]-np.floor((JJ-T0)/popt[0])

for i,dd in enumerate(JJ):
	print(dd, phase[i])
