# -*- coding: iso_8859_1 -*-

import numpy as np
import platform as pf

from ctypes import c_long, c_float, cdll
from numpy.ctypeslib import ndpointer

def add_sigma_clipping2(images, sigma, iterations):
	"""
	---------------------
	Purpose
	Combines a list of images with the sigma clipping method
	The method works pixel by pixel: for one pixel, it calculates the mean and std_deviation values on all images, then the pixel values deviating from the mean by sigma * std_deviation are rejected.
	The process is repeated iterations times, and the mean of the remaining pixel values is kept for the final image.
	The method is not recommended with a low number (<15) of images. For a higher number of images it is very efficient.
	---------------------
	Inputs
	* images (3D Numpy array) = input image data to be combined, as obtained by reading a list of image files with the function astro.get_imagedata().
	* sigma (float) = the pixel values that deviate from the mean value by more than sigma * std_deviation are rejected
	* iterations (integer) = for one pixel on all images, the process of calculating mean, std deviation and testing to reject pixel values is repeated iterations times. Once some images have been rejected for that pixel, the mean and std deviation change so it is possible to repeat the test.
	---------------------
	Output (2D Numpy array) = combined image
	---------------------
	"""
	ierror = 0
	while True:
		try:
			dim, dim_x, dim_y = images.shape

			indata = np.ascontiguousarray(np.array(images, dtype = np.float32))
			outdata = np.ascontiguousarray(np.empty((images.shape[1],images.shape[2]), dtype = np.float32))

			if pf.system()=='Windows':
				cdll.sigma_clipping.Sigma_clipping.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), c_float, c_long, ndpointer(dtype=np.float32)]
				cdll.sigma_clipping.Sigma_clipping(dim,dim_x,dim_y,indata,np.float32(sigma),iterations,outdata)
			elif pf.system()=='Linux':
				mydll = cdll.LoadLibrary("libsigmaclip.so")
				mydll.Sigma_clipping.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), c_float, c_long, ndpointer(dtype=np.float32)]
				mydll.Sigma_clipping(dim,dim_x,dim_y,indata,np.float32(sigma),iterations,outdata)

			del indata
			break
		except:
			print("error in aspylib.astro.add_sigma_clipping2()")
			ierror+=1
			if ierror>3:
				raise Exception("more than 3 consecutive failures, execution stopped")
			pass

	return outdata

def add_sigma_clipping(images, sigma, iterations, cutsize=[500,500]):
	"""
	---------------------
	Purpose
	Combines a list of images with the sigma clipping method
	This function must be used for very large images: it cuts the images into smaller subparts, and combines the subparts with astro.add_sigma_clipping2().
	---------------------
	Inputs
	* images (3D Numpy array) = input image data to be combined, as obtained by reading a list of image files with the function astro.get_imagedata().
	* sigma (float) = the pixel values that deviate from the mean value by more than sigma * std_deviation are rejected
	* iterations (integer) = the process of calculating mean, std deviation and testing to reject pixel values is repeated iterations times
	* cutsize ([integer, integer]) = 2 elements list giving the size of the largest subimage to be sent to astro.add_sigma_clipping2()
	---------------------
	Output (2D Numpy array) = combined image
	---------------------
	"""
	sx, sy= cutsize
	dim, dim_x, dim_y = images.shape

	nx = int(dim_x/sx) + 1
	ny = int(dim_y/sy) + 1
	
	result = np.zeros(np.shape(images)[1:3])

	for i in range(nx):
		for j in range(ny):
			i1 = i*sx
			i2 = min((i+1)*sx,dim_x)
			j1 = j*sy
			j2 = min((j+1)*sy,dim_y)
			result[i1:i2,j1:j2] = add_sigma_clipping2(images[:,i1:i2,j1:j2], sigma, iterations)

	return result

def add_mean(images):
	"""
	---------------------
	Purpose
	Combines a list of images by calculating the mean value of the pixels over the image list
	---------------------
	Inputs
	* images (3D Numpy array) = input image data to be combined, as obtained by reading a list of image files with the function astro.get_imagedata().
	---------------------
	Output (2D Numpy array) = average image
	---------------------
	"""
	return np.mean(images, axis=0)

def add_median2(images):
	"""
	---------------------
	Purpose
	Combines a list of images by calculating the median value of the pixels over the image list
	---------------------
	Inputs
	* images (3D Numpy array) = input image data to be combined, as obtained by reading a list of image files with the function astro.get_imagedata().
	---------------------
	Output (2D Numpy array) = combined median image
	---------------------
	"""

	ierror = 0
	while True:
		try:
			dim, dim_x, dim_y = images.shape

			indata = np.ascontiguousarray(np.array(images, dtype = np.float32))
			outdata = np.ascontiguousarray(np.empty((images.shape[1],images.shape[2]), dtype = np.float32))

			if pf.system()=='Windows':
				cdll.median.Median.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32)]
				cdll.median.Median(dim,dim_x,dim_y,indata,outdata)
			elif pf.system()=='Linux':
				mydll = cdll.LoadLibrary("libmedian.so")
				mydll.Median.argtypes = [c_long, c_long, c_long, ndpointer(dtype=np.float32), ndpointer(dtype=np.float32)]
				mydll.Median(dim,dim_x,dim_y,indata,outdata)

			del indata
			break
		except:
			print("error in aspylib.astro.add_median2()")
			ierror+=1
			if ierror>3:
				raise Exception("more than 3 consecutive failures, execution stopped")
			pass
			
	return outdata

def add_median(images, cutsize=[1000,1000]):
	"""
	---------------------
	Purpose
	Combines a list of images by calculating the median value of the pixels over the image list
	This function must be used for very large images: it cuts the images into smaller subparts, and combines the subparts with astro.add_median2().
	---------------------
	Inputs
	* images (3D Numpy array) = input image data to be combined, as obtained by reading a list of image files with the function astro.get_imagedata().
	* cutsize ([integer, integer]) = 2 elements list giving the size of the largest subimage to be sent to astro.add_median2()
	---------------------
	Output (2D Numpy array) = combined median image
	---------------------
	"""

	sx, sy= cutsize
	dim, dim_x, dim_y = images.shape

	nx = int(dim_x/sx) + 1
	ny = int(dim_y/sy) + 1

	result = np.zeros(np.shape(images)[1:3])

	for i in range(nx):
		for j in range(ny):
			i1 = i*sx
			i2 = min((i+1)*sx,dim_x)
			j1 = j*sy
			j2 = min((j+1)*sy,dim_y)
			result[i1:i2,j1:j2] = add_median2(images[:,i1:i2,j1:j2])

	return result
