# -*- coding: iso_8859_1 -*-

import numpy as np
from aspylib.astro import process2, process3

def mean_psf(image, nb_stars, fit_area, excluded=[], included=[], saturation = [False,0], text_display = True):
	"""
	---------------------
	Purpose
	Calculates the average parameters of the PSFs of the brightest stars on one image.
	The fits are performed with a 2D elliptical gaussian PSF.
	---------------------
	Inputs
	* image (2D Numpy array) = array that contains the data of one image (2D case) and for which the average PSF parameters must be computed.
	* nb_stars (integer, must be >1) = number of stars requested. The function is using a number of stars between 0.5*nb_stars and 1.5*nb_stars.
	* fit_area (integer, must be >0) = defines the +/- size of the square window used for star fitting (the square window has a size 2*fit_area+1).
	The window size must be large enough to include the stars for the PSF fit, but small enough not to include contributions from other stars.
	* excluded (list) = list of 4 elements lists, with the form [[x11,x12,y11,y12], ..., [xn1,xn2,yn1,yn2]]. Each element is a small part of the image that will not be used for star detection.
	This optional parameter allows to reject problematic areas in the image (e.g. saturated star(s), artefact(s), moving object(s)..)
	* included (list) = list of 4 integers, with the form [x1,x2,y1,y2]. It defines a subpart of the image where all the stars will be detected.
	* saturation (list) = optional argument. Two elements lists: if saturation[0] is True, the objects with maximum signal >= saturation[1] are disregarded.
	* text_display (boolean) = optional argument. If True then some information about the star detection is printed in the console.
	---------------------
	Output (list) = list with 3 elements, in the form [fhwm_small, fwhm_large, angle]. The list elements are respectively:
	- fwhm_small is the average smallest full width half maximum of the elliptical gaussian PSF in pixels
	- fwhm_large is the average largest full width half maximum of the elliptical gaussian PSF in pixels
	- angle is the average angular direction of the largest fwhm, measured clockwise starting from the vertical direction and expressed in degrees. The average direction of the smallest fwhm is obtained by adding 90 deg to angle.
	---------------------
	"""
	
	#fitting stars
	stars = process2.find_stars(image, nb_stars, excluded=excluded, included=included, detection_area = 2, saturation = saturation, margin=fit_area, text_display = text_display)
	stars = process3.fit_stars(image, stars, fit_area, maxshift = 2, error_messages = 1, full_info=True)

	#picking information
	fwhm_1 = []
	fwhm_2 = []
	cosine = []
	sine = []

	for xy in stars:
		fwhm_1.append(xy[5])
		fwhm_2.append(xy[6])
		cosine.append(np.cos(xy[7]))
		sine.append(np.sin(xy[7]))
		
	return [np.nanmean(fwhm_1), np.nanmean(fwhm_2), np.arctan2(np.nanmean(sine),np.nanmean(cosine))]
