# -*- coding: iso_8859_1 -*-

import sys
import numpy as np
import matplotlib.pyplot as plt

from scipy import optimize
from aspylib import astro
from aspylib.astrometry import astrometry2, astrometry3, astrometry5

def matching_two_images(image1, image2, distcrit = 5, nbdetstars = [60,200], text_display = True, plotting=False): 
	"""
	---------------------
	Purpose
	Find a linear transformation (translation + scaling + rotation) that allows to accurately match two images. The pixel coordinates of one star on image1, after transformation, point to the same star on image2.
	Two iterations are needed to get accurate results, since no a priori information on the transformation is provided. 
	First an approximate law is computed (4 coefficients such that X2=scaling*(X1*cos(phi)-Y1*sin(phi))+dx, Y2=scaling*(X1*sin(phi)+Y1*cos(phi))+dy) based on a cross-correlation method.
	Then pairs of stars are identified between both images and an accurate law is computed (6 coefficients such that X2 = a0+a1*X1+a2*Y1, Y2 = a3+a4*X1+a5*Y1).
	The algorithm works only if the rotation angle between image1 and image2 is smaller than +/- 90 deg : if one image is flipped (180 deg rotation) it fails.
	---------------------
	Inputs
	* image1 (2D Numpy array) = image data for image 1.
	* image2 (2D Numpy array) = image data for image 2.
	* distcrit (integer) = optional argument, default value is 5. This parameter represents a distance threshold, in pixel. If one star taken on image1, after the transformation, is closer than distcrit from a star on image2, then both are assumed to be two images of one single star for calculating the accurate transformation law.
	* nbdetstars (list with two integers) = optional argument, default value is [60,200]. nbdetstars[0] (resp. nbdetstars[1]) is the number of stars detected on both images for computing the approximate (resp. accurate) transformation law. If the function fails to calculate the matching, in some cases this may be fixed by setting nbdetstars[0] and nbdetstars[1] to different values.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	* plotting (boolean) = optional argument. If equal to True, the script is paused to display a plot showing the matching between stars from image1 and image2. If equal to False (default) no plot is shown.
	---------------------
	Output (list) = List of transformation coefficients [[scaling, phi, dx, dy], [a0,a1,a2,a3,a4,a5]] for both the approximate and accurate laws.
	Note: the rotation angle phi is expressed in radians.
	---------------------
	"""
	if text_display:
		print("")
		print("aspylib.astrometry.matching_two_images():")

	#*********************************************************************************************************
	#		(1) preparing inputs
	#*********************************************************************************************************

	#--- finding stars on images ---
	stars1 = astro.find_stars(image1, max(nbdetstars), saturation = [False,0], text_display = text_display)
	stars1 = astro.centroid_stars(image1, stars1, 2)
	stars1, indices = astro.remove_duplicates(stars1)
	stars1low = stars1[:nbdetstars[0],:]
	stars1high = stars1[:nbdetstars[1],:]

	stars2 = astro.find_stars(image2, max(nbdetstars), saturation = [False,0], text_display = text_display)
	stars2 = astro.centroid_stars(image2, stars2, 2)
	stars2, indices = astro.remove_duplicates(stars2)
	stars2low = stars2[:nbdetstars[0],:]
	stars2high = stars2[:nbdetstars[1],:]

	#*********************************************************************************************************
	#		(2) approximate transformation
	#*********************************************************************************************************

	#--- approximate solution ---
	#scaling + rotation + translation that transform the pixel coordinates into stereographic coordinates
	scaling, phi, dx, dy = astrometry2.approximate_astrometry(stars1low, stars2low, nb_iterations=2, resolution=1024)
	newcoef = [[scaling, phi, dx, dy],[]]		
	p0 = [dx, scaling*np.cos(phi), -scaling*np.sin(phi), dy, scaling*np.sin(phi), scaling*np.cos(phi)]
	
	#*********************************************************************************************************
	#		(3) checking + display results
	#*********************************************************************************************************
	
	#--- star transformation ---
	trans1low = astrometry5.transform_stars(newcoef[0], stars1low)
	#--- calculate matching pairs ---
	x,y = astrometry3.matching_pairs(trans1low,stars2low,distcrit)

	#--- checking ---
	if len(x)<10:
		print("")
		print("aspylib.astrometry.matching_two_images():")
		print("-----------------------------------------------------------------------")
		print("astrometric reduction failed, with only "+str(len(x))+" matching stars")
		print("-----------------------------------------------------------------------")
		sys.exit("Script execution stopped")
		
	if text_display:
		print("")
		print("--- searching approximate astrometry ---")
		print("nb of stars used (image1 / image2):",len(stars1low),"/",len(stars2low))
		print("approximate astrometric coefficients:", scaling, phi, dx, dy)
		print("nb of matched pairs:" ,len(x))
	
	if plotting:
		plt.figure()
		plt.scatter(trans1low[:,1],trans1low[:,0],s=20)
		plt.scatter(stars2low[:,1],stars2low[:,0],s=20,marker='+',color='r')
		plt.grid()
		plt.title("first star matching")
		plt.show()

	#*********************************************************************************************************
	#		(4) accurate astrometry
	#*********************************************************************************************************

	#--- star transformation ---
	trans1high = astrometry5.transform_stars(newcoef[0], stars1high)
	#--- calculate matching pairs ---
	x,y = astrometry3.matching_pairs(trans1high,stars2high,distcrit)

	#--- accurate astrometry ---
	if len(x)<20:
		print("")
		print("aspylib.astrometry.matching_two_images():")
		print("-----------------------------------------------------------------------")
		print("searching accurate astrometry:")
		print("not enough matching pairs for least square fit, an error will occur")
		print("-----------------------------------------------------------------------")
	coordinates = np.vstack([stars1high[x,0], stars1high[x,1], stars2high[y,0], stars2high[y,1]]).transpose()
	errfunc = lambda p, xy: (astrometry5.distorsion_law_linear(p, xy[:,:2]) - xy[:,2:]).flatten() # Distance to the target function
	p1, success = optimize.leastsq(errfunc, p0, args=(coordinates))
	newcoef[1] = p1
		
	#*********************************************************************************************************
	#		(5) display results
	#*********************************************************************************************************

	if text_display:
		print("")
		print("--- searching accurate astrometry ---")
		print("nb of stars used (detected / from catalog):",len(stars1high),"/",len(stars2high))
		print("nb of matched pairs used for accurate astrometry:" ,len(x))
		print("astrometric coefficients:", p1)

	if plotting:
		plt.figure()
		plt.scatter(trans1high[:,1],trans1high[:,0],s=20)
		plt.scatter(stars2high[:,1],stars2high[:,0],s=20,marker='+',color='r')
		plt.grid()
		plt.title("accurate star matching")
		plt.show()

	return newcoef
