# -*- coding: iso_8859_1 -*-

import warnings
import numpy as np
from astropy.io.votable import parse

def read_votable(filename, ordering = [False,''], remove_duplicates = [False, []], text_display=True):
	"""
	---------------------
	Purpose
	Read out a xml file with VOtable format, that was obtained with the Vizier or SkyBoT web service
	---------------------
	Inputs
	* filename (string) = specifies the xml file from which the data must be read out.
	It can be a full filename with path information, or a simple filename in which case the active directory is used.
	* ordering (list) = optional parameter, default is [False, '']. If ordering[0] is True, all the astronomical objects contained in the data are ordered in increasing order using the field ordering[1].
	For instance, if ordering = [True, 'B1mag'] then the output data is such that data['B1mag'] is in increasing order.
	* remove_duplicates (list) = optional parameter, default is [False,[]].
	If remove_duplicates[0] is True, then all astronomical objects having two or more occurrences with the same fields remove_duplicates[1][i] (with i=1,..,len(remove_duplicates[1])) are identified and all redundant occurrences are deleted.
	For instance, remove_duplicates = [True, ['RAJ2000', 'DEJ2000']] allows to make sure that we have only one object per (alpha, delta) position.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	---------------------
	Output (Numpy record array) = array that contains all the data. The columns get their names from both the ID and name attributes of the FIELD elements in the VOTABLE file.
	See online help: https://docs.astropy.org/en/stable/io/votable/index.html and http://www.ivoa.net/documents/VOTable/.
	---------------------
	"""

	#------ (1) Converting votable to Numpy array -----------------
	if text_display:
		print("")
		print("aspylib.astrometry.read_votable():")
		print("--- reading .xml catalog ---")

	warnings.simplefilter("ignore")
	votable = parse(filename, pedantic=False)
	warnings.resetwarnings() 

	#------ added 17-12-2012 to avoid error with VO.Table saying  "No solar system object was found in the requested FOV" ------
	nbtables = 0
	for table in votable.iter_tables():
		nbtables = 1
		break
	if nbtables==0:
		return []
	#------------

	result = votable.get_first_table()
	data = result.array
	
	#print(data.dtype.names)
	#print(data)
	#print("")

	#------ (2) Ordering objects -----------------

	if ordering[0]:
		data = data[np.argsort(data[ordering[1]])]

	#------ (3) Removing duplicates -----------------

	if remove_duplicates[0]:
		data2 = data[remove_duplicates[1][0]]
		for i in range(1,len(remove_duplicates[1])):
			if i==1:
				data2 = np.vstack([data2, data[remove_duplicates[1][i]]]).transpose()
			else:
				data2 = np.vstack([data2.transpose(), data[remove_duplicates[1][i]]]).transpose()
		data2 = np.core.records.fromarrays(list(data2.transpose()))
		data2, indices = np.unique(data2, return_index=True)
		indices = np.sort(indices)
		data = data[indices]

	return data
