# -*- coding: iso_8859_1 -*-

import numpy as np
from aspylib import astro, conf
from aspylib.calib import calib3

def calibrate(images, darkframe, flatframe):
	"""
	---------------------
	Purpose
	Performs a radiometric calibration based on the formula:
		images_calib = (images - darkframe) / flatframe
	Also performs the following corrections:
	- the value of the calibrated image at the positions of the brightest hot pixels is replaced by the average of the calibrated image at the 4 nearest neighbours (x+1,y), (x-1,y), (x,y+1), (x,y-1).
	- the calibrated images are set to conf.saturation if (1) the initial images are equal to conf.saturation ; or if (2) the calibrated image is higher than conf.saturation.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = image data, 2D Numpy array for a single image, or 3D Numpy array for a list of images. Contains the uncalibrated images.
	* darkframe (2D Numpy array) = image data, contains the dark master frame. The dark frame has been exposed in the same conditions (temperature, exposure time) as the uncalibrated images, in the absence of light.
	* flatframe (2D Numpy array) = image data, contains the flat master frame. This image provides the information on the system "radiometric gain", e.g. describing vignetting effects and pixel response non-uniformities, etc. The flat frame is typically obtained by subtracting 2 images A-B: A is obtained by looking at a bright and uniform scene, and B is obtained with the same integration time in the absence of signal.
	---------------------
	* images_calib (2D or 3D Numpy array) = image data, 2D Numpy array for a single image, or 3D Numpy array for a list of images. Contains the radiometrically calibrated images.
	---------------------
	"""
	#---------------------------------
	#possible issues/improvements:
	#(1) it may be that new saturated pixels appear on the calibrated image. Any possibility to minimize/control/report them ?
	#---------------------------------

	#------ detecting hot pixels ------
	hotpixels = np.tile(calib3.find_hotpixels(darkframe, 100),(1,1,1))

	#------ rescaling flat frame ------
	flatframe = flatframe * 1.2/ astro.get_median(flatframe)
	flatframe = flatframe + 1e-10

	#------ images calibration ------
	images_calib = (images - darkframe) / flatframe
	
	#correcting the hot pixels
	images_avg = (np.roll(images_calib,1,axis=1) + np.roll(images_calib,-1,axis=1) + np.roll(images_calib,1,axis=2) + np.roll(images_calib,-1,axis=2)) / 4.
	images_calib = images_avg*hotpixels + images_calib*(1.-hotpixels)

	#checking for saturation
	test = (images_calib<conf.saturation-0.001)*(images!=conf.saturation)
	images_calib = (1.-test)*conf.saturation + test*images_calib

	return images_calib
