# -*- coding: iso_8859_1 -*-

import numpy as np
from aspylib import astro

def coalign(images, fit_area=10, nb_stars=12, excl=[], incl=[], text_display=False, return_shifts=False):
	"""
	---------------------
	Purpose
	Given a series of FITS images, co-align all images with the first one by computing and applying the proper translations.
	The algorithm for the computation of the image shifts is based on (1) detection of a small number of stars (default is 12 stars) (2) matching the positions of the stars between pairs of images.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = image data, 2D Numpy array for a single image, or 3D Numpy array for a list of images. Contains the images to be co-aligned.
	* fit_area (integer, must be >0) = optional argument, default value is 10. Defines the +/- size of the square window, measured in pixels, used for star fitting. The square window has a size 2*fit_area+1. The window size must be large enough to include the stars for the PSF fit, but small enough not to include contributions from other stars.
	* nb_stars (integer) = optional argument, default value is 12. Gives the number of stars detected on each images and used for computing the shifts.
	* excluded (list) = optional argument. List of 4 elements lists, with the form [[x11,x12,y11,y12], ..., [xn1,xn2,yn1,yn2]]. Each element is a small part of the image that will be excluded for the star detection. This optional parameter allows to reject problematic areas in the image (e.g. saturated star(s), artefact(s), moving object(s)..)
	* included (list) = optional argument. List of 4 integers, with the form [x1,x2,y1,y2]. It defines a subpart of the image where all the stars will be detected.
	Both excluded and included can be used simultaneously.
	* text_display (boolean) = optional argument. If equal to True, some information about the image shift computation is written in the console window. If equal to False (default) nothing is written.
	* return_shifts (boolean) = optional argument, default value is False. If True, then the function returns the coaligned images + the shifts. If False, the function only returns the coaligned images.
	---------------------
	Outputs = images_aligned, shifts
	* images_aligned (2D or 3D Numpy array) = image data, 2D Numpy array for a single image, or 3D Numpy array for a list of images. Contains the co-aligned images.
	* shifts (2D Numpy array) = only returned if return_shifts=True. Provides the (dx,dy) shifts of all images with respect to the first one. Has the form [[dx1,dy1],[dx2,dy2], ..., [dxn,dyn]] with [dx1,dy1]=[0.0,0.0].
	---------------------
	"""
	print("")
	print("aspylib.calib.coalign():")

	images_aligned = np.array(images)
	print("")
	print('shifts of image 1 wrt image 1 (vert, hor):', 0., 0.)

	stars0 = astro.find_stars_accurate(images[0,:,:], nb_stars, fit_area, excluded=excl, included=incl, filter=1)
	shifts = []
	for i in range(1,len(images)):

		#calculate shifts
		starsi = astro.find_stars_accurate(images[i,:,:], nb_stars, fit_area, excluded=excl, included=incl, filter=1)
		dx,dy = astro.find_shift_stars_matching(stars0, starsi, text_display=text_display)
		shifts.append([dx,dy])
		print('shifts of image', str(i+1),' wrt image 1 (vert, hor):', dx, dy)

		#shift image
		images_aligned[i,:,:] = astro.translate_Bspline(images[i,:,:], (-dx,-dy), 3)
	
	if return_shifts:
		shifts = np.array(shifts)
		return images_aligned, shifts
	else:
		return images_aligned

def coalign2(images, return_shifts=False):
	"""
	---------------------
	Purpose
	Given a series of FITS images, co-align all images with the first one by computing and applying the proper translations.
	The image shifts are calculated from the cross-correlation of each ith image with the 1st image in the series.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = image data, 2D Numpy array for a single image, or 3D Numpy array for a list of images. Contains the images to be co-aligned.
	* return_shifts (boolean) = optional argument, default value is False. If True, then the function returns the coaligned images + the shifts. If False, the function only returns the coaligned images.
	---------------------
	Outputs = images_aligned, shifts
	* images_aligned (2D or 3D Numpy array) = image data, 2D Numpy array for a single image, or 3D Numpy array for a list of images. Contains the co-aligned images.
	* shifts (2D Numpy array) = only returned if return_shifts=True. Provides the (dx,dy) shifts of all images with respect to the first one. Has the form [[dx1,dy1],[dx2,dy2], ..., [dxn,dyn]] with [dx1,dy1]=[0.0,0.0].
	---------------------
	"""
	print("")
	print("aspylib.calib.coalign2():")

	images_aligned = np.array(images)
	print("")
	print('shifts of image 1 wrt image 1 (vert, hor):', 0., 0.)
	
	shifts = []
	for i in range(1,len(images)):

		#calculate shifts
		dx, dy = astro.find_shift_crosscorrelation(images[0,:,:], images[i,:,:])
		shifts.append([dx,dy])
		print('shifts of image', str(i+1),' wrt image 1 (vert, hor):', dx, dy)

		#shift image
		images_aligned[i,:,:] = astro.translate_Bspline(images[i,:,:], (-dx,-dy), 3)
		
	if return_shifts:
		shifts = np.array(shifts)
		return images_aligned, shifts
	else:
		return images_aligned
