# -*- coding: iso_8859_1 -*-

from aspylib import astro

def process_darkframe_offsetframe(Imagelist, Finalimage, Config, Headers={}):
	"""
	---------------------
	Purpose
	Computes a master dark or master offset image from a series of dark or offset FITS images with median co-addition, and saves it to a FITS file.
	The script assumes that the same integration time was used for all the images in the series.
	---------------------
	Inputs
	* Imagelist (list) = Dark or offset images to be co-added. Has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	* Finalimage (list) = one element list containing a string [string]. It gives the complete filename (including path) of the FITS file where the final dark or offset image will be saved.
	* Config (integer) = the FITS headers of the final image are copy-pasted from the first image in Imagelist. Config defines which set of rules must be used, additionally, to modify the FITS headers obtained from the image Imagelist[0]. The file conf.py allows to define several sets of rules and specifies which headers must be removed, kept, or added.
	* Headers (dictionary) = optional argument. Default value is an empty dictionary. Gives additional header entries, to be added to the headers of the final image. If one keyword already exists in the headers of Imagelist[0], it will be overwritten by the new value/comment provided with Headers.
	FITS headers are created in three steps in this order:
		(1) headers are read-out from the image Imagelist[0] and modified according to the parameter Config,
		(2) additional header entries are added according to Headers,
		(3) as a last step, some modifications are done for the keywords DATE-OBS and DATE-END: see Python code.
	---------------------
	Output (2D Numpy Array) = data of the final dark or offset image.
	---------------------
	"""

	#--- loading and computing final frame ---
	image_final = astro.add_median(astro.get_imagedata(Imagelist))
	
	#--- creating headers ---
	headerdict = astro.create_headers(Config, [Imagelist[0]])[0]
	headerdict.update(Headers)
	Dateobs = sorted(astro.get_headervalues(Imagelist, 'DATE-OBS'))[0]
	Dateend = sorted(astro.get_headervalues(Imagelist, 'DATE-END'))[-1]
	headerdict.update({'DATE-OBS':[Dateobs,''],'DATE-END':[Dateend,'']})
	
	#--- saving the image ---
	astro.save_imagelist(image_final, [headerdict], [Finalimage])
	print("")
	print("aspylib.calib.process_darkframe_offsetframe():")
	print(Finalimage + ' saved')
		
	return image_final