# -*- coding: iso_8859_1 -*-

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.ticker as tkr

from scipy import interpolate
from aspylib import astro, conf
from aspylib.cdr import cdr1

def load_magdata(Filelist, zero_mean=True):
	"""
	---------------------
	Purpose
	Loads a list of lightcurve files, and returns a list of Numpy arrays:
	- each array corresponds to a lightcurve file,
	- the Numpy arrays have 3 columns
	- the first column contains Julian dates of the observations,
	- the second one contains the differential magnitudes of the target (asteroid or star),
	- the third one contains an estimation of the magnitude noise for each magnitude value.
	---------------------
	Definitions
	- DIFFERENTIAL MAGNITUDES are not true absolute magnitudes (the difference being an unknown offset),
	- MEASURED MAGNITUDES are the magnitude values obtained by aperture photometry on each images,
	- diff. magnitudes are obtained as the difference between meas. magnitudes for the target and the average meas. magnitudes of all comparison stars.
	---------------------
	Inputs
	* Filelist (list) = has the form [string_1, string_2, ..., string_n].
	Each string gives the complete filename (including path) of a file that contains the lightcurve measurements.
	The file format is the following:
	- the file contains one header part, followed by one data part
	- the format of the header and data parts must follow the CourbRot conventions (as described here: http://obswww.unige.ch/~behrend/page_fmt.html)
	- the header keywords FMT, NOM, MES, POS are mandatory
	- the header keyword FMT must contain one "D" and one "V". For instance we can have "FMT xDVRRR". 
	- the header keyword POS indicates if dates are given for start of exposure (POS -1 xx), mid-exposure (POS 0 xx) or end of exposure (POS +1 xx) with xx being the exposure time in seconds
	* zero_mean (boolean) = optional argument, default value is True. If True all the individual lightcurves are shifted so that their mean value is 0. If False, the mean is not modified.
	---------------------
	Remarks
	- one file can contain several nights of observation, but must correspond to a single set of comparison stars
	---------------------
	Output (list) = list of 3 columns Numpy arrays, as described above.
	---------------------
	"""
	print("")
	print("aspylib.cdr.load_magdata():")
	print("------ loading measured data ------")

	alldata = []

	for ff in Filelist:
		data, offset = cdr1.load_magfile(ff)
		
		#checking reference stars (if there are ref stars !)
		#if np.shape(data)[1]>2:
		#	plt.figure()
		#	for i in range(np.shape(data)[1]-2):
		#		refstar = data[:,i+2]-np.mean(data[:,2:],axis=1) 
		#		plt.plot(refstar-np.mean(refstar)+0.02*np.double(i),label=str(i+1))
		#	plt.legend()
		#	plt.grid(True)
		#	ax = plt.gca()
		#	ax.set_ylim(ax.get_ylim()[::-1])
		#	plt.title("checking reference stars")
		#	plt.show()

		#converting date format if needed
		if data[0,0]>10000000.:
			data[:,0] = np.array(astro.dates_to_juliandays(list(data[:,0])))

		#shifting date to mid-exposure
		data[:,0] = data[:,0] + offset/86400.

		#calculating differential magnitude
		if np.shape(data)[1]>2:
			data[:,1] = data[:,1] - np.mean(data[:,2:],axis=1)
			data = data[:,:2]
			
		if zero_mean:
			#setting average magnitude to 0
			data[:,1] = data[:,1] - np.mean(data[:,1])
		
		alldata.append(data)
	
	print("")
	print("------ evaluating noise in measurements ------")

	for i,dd in enumerate(alldata):
		noise = cdr1.calc_noise(dd[:,1])
		noise = np.tile(dd[:,0]*0.+noise,(1,1))
		alldata[i] = np.concatenate((dd.transpose(),noise)).transpose()

	return alldata

def plot_magdata(alldata, title, dmag = -1, dcolor = 0):
	"""
	---------------------
	Purpose
	Plots the differential magnitude data, as returned by the function load_magdata(), as a function of Julian date.
	---------------------
	Inputs
	* alldata (list) = list of Numpy arrays containing measured lightcurve data:
	- each array has 3 columns
	- the first column contains Julian dates of the observations,
	- the second one contains the differential magnitudes of the target (asteroid or star),
	- the third one contains an estimation of the magnitude noise for each magnitude value.
	* title (string) = gives the title of the plot.
	* dmag (float) = optional argument, default value is -1. If negative or equal to zero, dmag is not used. If positive and different from zero, dmag is the +/- range of the vertical (magnitude) axis used in the plot.
	* dcolor (integer) = optional argument. When the plots are created, the colors used for the curves are picked up in the list conf.col defined by AsPyLib. 
	dcolor is the index of the first color that is used. When it is equal to 0 (default value), the first color in the list is used for the first curve.
	---------------------
	Output = no output.
	---------------------
	"""
	plt.figure()
	for i,dd in enumerate(alldata):
		plt.plot(dd[:,0],dd[:,1],c=conf.col[(dcolor+i)%16])
	plt.title(title)
	plt.grid()
	if dmag>0:
		plt.ylim([np.mean(dd[:,1])-dmag,np.mean(dd[:,1])+dmag])
	ax = plt.gca()
	ax.set_ylim(ax.get_ylim()[::-1])
	ax.xaxis.set_major_locator(tkr.MaxNLocator(4))
	dx = ax.xaxis.get_ticklocs()[1]-ax.xaxis.get_ticklocs()[0]
	if ((dx%0.01)>1.e-6)and((dx%0.01)<0.00999999):
		ax.xaxis.set_major_formatter(tkr.FormatStrFormatter('%.3f'))
	elif ((dx%0.1)>1.e-6)and((dx%0.1)<0.0999999):
		ax.xaxis.set_major_formatter(tkr.FormatStrFormatter('%.2f'))
	else:
		ax.xaxis.set_major_formatter(tkr.FormatStrFormatter('%.1f'))
	plt.ylabel("differential magnitude")
	plt.xlabel("Julian date (UT, no time correction)")
	plt.show()

	
def time_correction(alldata, target, argument):
	"""
	---------------------
	Purpose
	Performs time corrections to a list of measured lightcurves.
	The corrections depend on the nature of the variable object that is observed:
	- if the variable object is an asteroid the time delay due to light propagation from asteroid to Earth is subtracted from the Julian dates of the observations
	- if the variable object is a variable star the Julian dates are converted to Heliocentric Julian dates
	---------------------
	Inputs
	* alldata (list) = list of Numpy arrays containing measured lightcurve data:
	- each array has 3 columns
	- the first column contains Julian dates of the observations,
	- the second one contains the differential magnitudes of the target (asteroid or star),
	- the third one contains an estimation of the magnitude noise for each magnitude value.
	* target (integer) = 1 for asteroid, 2 for star
	* argument (string or [float,float])
	- if target = 1 (asteroid) 
	Then argument is a string giving the complete filename (including path) of a file that contains the distance Earth-asteroid as a function of time.
	--> the file starts immediately with numeric data. The data is provided in the form of 2 columns separated with a tabulation.
	--> first column contains Julian dates
	--> second column contains the Earth-asteroid distances, in astronomical units (AU)
	- if target = 2 (star)
	Then argument is a list [float, float], the two float numbers being respectively the right ascension (deg) and declination (deg) of the star target.
	Using these coordinates, the Julian dates are converted to Heliocentric Julian dates.
	---------------------
	Output (list) = list of time corrected measured lightcurves.
	---------------------
	"""
	print("")
	print("aspylib.cdr.time_correction():")

	if target == 1:

		print("")
		print("------ subtracting time delay due to light propagation from asteroid to Earth ------")

		sec_per_au = 149598000. / 299792.458
		dist = np.loadtxt(argument)
		cubic_interp = interpolate.interp1d(dist[:,0], dist[:,1], kind='cubic')

		for i,dd in enumerate(alldata):
			dist_Earth = cubic_interp(dd[:,0])
			alldata[i][:,0] = dd[:,0] - dist_Earth * sec_per_au / 86400.

	elif target ==2:

		print("")
		print("------ converting Julian Dates to Heliocentric Julian Dates ------")

		#---------------- References ---------------------------------------------------------------------------------------------------------------#
		#	J.Meeus = "Calculs astronomiques a l'usage des amateurs", SAF (1986)																	#
		#	A.U.Landolt, K.L.Blondeau = "The calculation of heliocentric corrections", Pub. Astron. Soc. Pacific., vol 84 (Dec 1972), pp 784-809	#
		#-------------------------------------------------------------------------------------------------------------------------------------------#
		
		for i,dd in enumerate(alldata):
			T = (dd[:,0] - 2415020.)/36525.
			L = 279.69668 + 36000.76892*T + 0.0003025*T*T
			M = 358.47583 + 35999.04975*T - 0.000150*T*T - 0.0000033*T*T*T
			e = 0.01675104 - 0.0000418*T - 0.000000126*T*T
			C = (1.919460- 0.004789*T - 0.000014*T*T)*np.sin(M*np.pi/180.) + (0.020094 - 0.000100*T)*np.sin(2.*M*np.pi/180.) + 0.000293*np.sin(3.*M*np.pi/180.)
			Lv = L+C
			v = M+C
			R = 1.0000002*(1.-e*e)/(1.+e*np.cos(v*np.pi/180.))
			epsilon = 23.452294- 0.0130125*T - 0.00000164*T*T + 0.000000503*T*T*T

			correction = R*np.cos(Lv*np.pi/180.)*np.cos(argument[0]*np.pi/180.)*np.cos(argument[1]*np.pi/180.) \
						+ R*np.sin(Lv*np.pi/180.)*(np.sin(epsilon*np.pi/180.)*np.sin(argument[1]*np.pi/180.) \
									+np.cos(epsilon*np.pi/180.)*np.cos(argument[1]*np.pi/180.)*np.sin(argument[0]*np.pi/180.))
			
			alldata[i][:,0] = dd[:,0] - 0.0057755 * correction
		
	return alldata

