# -*- coding: iso_8859_1 -*-
"""
	module where aspylib settings are defined.
"""

import platform as pf

#------------------------------------------------------------------------------------------
#	Notes about FITS image levels (taken from https://fits.gsfc.nasa.gov/fits_standard.html)
#------------------------------------------------------------------------------------------
#
# BITPIX can take the following values:
# 8 --> unsigned 8-bits integers in the range [0, 255]
# 16 --> signed 16-bits integers in the range [-32768, +32767]
# 32 --> signed 32-bits integers in the range [-2147483648, +2147483647]
# 64 --> signed 64-bits integers in the range [-9223372036854775808, +9223372036854775807] --> new format approved in Dec. 2005
# -32 --> 32-bit IEEE float
# -64 --> 64-bit IEEE float
#
# The standard format is BITPIX=16. The PRISM software also allows to manipulate images with BITPIX=-32.
#
# Unsigned integers
#"The FITS format does not support a native unsigned integer data type (except for the unsigned 8-bit byte data type) therefore unsigned 16-bit, 32-bit, or 64-bit binary integers cannot be stored directly in a FITS data array. Instead, the appropriate oﬀset must be applied to the unsigned integer to shift the value into the range of the corresponding signed integer, which is then stored in the FITS ﬁle. The BZERO keyword shall record the amount of the oﬀset needed to restore the original unsigned value. The BSCALE keyword shall have the default value of 1.0 in this case, and the appropriate BZERO value, as a function of BITPIX, is speciﬁed in Table 11."
#
# BITPIX       Native type       Physical type        BZERO
# 8            unsigned          signed 8-bits        -128 (-2^7)
# 16           signed            unsigned 16-bits     32768 (2^15)
# 32           signed            unsigned 32-bits     2147483648 (2^31)
# 64           signed            unsigned 64-bits     9223372036854775808 (2^63)
#
# See also: 
# https://fits.gsfc.nasa.gov/fits_standard.html
# http://www.sal.wisc.edu/~khn/salt/Outgoing/documentation/PySALT/The_PyFITS_Handbook.pdf
#------------------------------------------------------------------------------------------
#	constants
#------------------------------------------------------------------------------------------

#saturation level. This parameter is updated and possibly modified each time an image is loaded
saturation = 65535.

#default saturation level for 32-bit or 64-bit IEEE float images
saturation_default_real = 65535.

#default working folder
folder = 'C:\\Users\\caronjf\\Desktop\\Perso\\Aspylib_work\\'
if pf.system()=='Linux':
	folder = "/tmp/"

#filename separator
sep = '\\'
if pf.system()=='Linux':
	sep = "/"

#color for plotting
col = ['b', 'g', 'r', 'c', 'm', 'y', 'k', '#00FF26', '#FEAE02', '0.66', '0.33', '#006FFF', '#FE80F5', '#BE0D00', '#F3EB00', '#FF2E89']
col2 = [col[i] for i in [0,11,3,7,1,5,14,8,2,15,12,4,13,6,10,9]]

#------------------------------------------------------------------------------------------
#	default headers
#------------------------------------------------------------------------------------------

def set_headers(Config):
	"""
	---------------------
	Purpose
	This function defines a set of rules to generate FITS headers for new images, from the headers of existing images.
	It is located in the file conf.py so it can be easily edited by the user.
	It is called by: create_headers(Config, Imagelist).
	---------------------
	Parameters to be modified by user
	* take_imageheaders (boolean) = defines if the headers from the images are kept (or not) when creating the header list in Python.
	* hd_add (dictionary) = list of header keywords. The keywords are added to the Python header list, with their corresponding values and comments. If no value/comment are provided in Python (empty list []), then they are taken from the image.
	If no value/comment are provided in Python (empty list []) and additionaly the keywords are absent from the image, they are not added.
	* hd_del (dictionary) = list of header keywords. They are removed from the Python header list.
	---------------------
	"""
	
	hd_add = {}
	hd_del = {}
	take_imageheaders = False
	
	if Config == 0: 	#takes all the headers from the images without any modification
		take_imageheaders = True
		
	if Config == 1: 	#takes only the following headers from the images
		take_imageheaders = False
		hd_add = {'BINX':[],'BINY':[],'BIN1':[],'BIN2':[],'DATE-OBS':[],'DATE-END':[],'EXPTIME':[],'EXPOSURE':[],'GPS-DATE':[],'OBJECT':[]}

	if Config == 2: 	#ST-8300 + ED 80/600 from Dauban
		take_imageheaders = False
		hd_add = {'BINX':[],'BINY':[],'BIN1':[],'BIN2':[],'DATE-OBS':[],'DATE-END':[],'EXPTIME':[],'EXPOSURE':[],'GPS-DATE':[],'OBJECT':[]}
	
		hd_add.update({'OBSERVER':	['F.Kugel, J.Caron','Observer name']})
		hd_add.update({'ORIGIN':		['UAI A77 Observatoire Chante-Perdrix, Dauban','Origin place of FITS file']})
		hd_add.update({'SITELAT':		['43:59:59.0 North','Latitude where data acquisition was made']})
		hd_add.update({'SITELONG':	['05:38:51.0 East','Longitude where data acquisition was made']})
		hd_add.update({'CAMERA':		['ST-8300','Data acquisition camera']})
		hd_add.update({'TELESCOP':	['ED 80/600','Data acquisition telescope']})
		hd_add.update({'DIAMETER':	[80.,'Telescope aperture diameter (mm)']})
		hd_add.update({'FOCAL':		[600.,'Telescope focal length (mm)']})

	if Config == 3:		#remove some headers from the images
		take_imageheaders = True
		hd_del = {'DATAMIN':[],'DATAMAX':[]}

	#here create more configurations as needed
	#...
	
	return take_imageheaders, hd_add, hd_del
