# -*- coding: iso_8859_1 -*-

import numpy as np
import matplotlib.pyplot as plt

def compute_statistics(mag, nbref = 20, plotting = False):
	"""
	---------------------
	Purpose
	Given a set of magnitude measurements for n objects on i images with p sets of photometric disks, this function performs the following tasks without user intervention:
	- filtering of measurements with bad quality (>20% of magnitudes are np.nan, checked for each object/disks combinations over all images)
	- identifying candidate reference stars (stars with all their measurements correct)
	- identifying "nbref" best reference stars (stars which have the lowest noise + the lowest amplitude between min and max magnitudes)
	- computing for all objects the differential magnitudes and their noise characteristics
	- correcting for the systematic difference (offset) between different photometry disks. The offsets are due to the impact of having a different photometry disk + having a different set of optimal reference stars
	- for each star, computing the optimal set of photometry disks
	Then the function returns for each object:
	- the best differential magnitude
	- the mean magnitude, RMS magnitude noise, RMS magnitude noise after filtering out slow variations
	- a flag indicating the quality of the magnitude data
	- which set of photometry disks has been used
	Plus, the function returns:
	- the best set of reference stars identified for each set of photometry disks
	Requirements:
	- measurements of magnitudes on at least 5 images (to compute magnitude noise after removing slow variations)
	- at least nbref objects with clean (no np.nan) magnitudes to be used as reference stars. If there are less clean objects then all of them will be used as reference stars.
	- this function only measures static (non-moving) objects, so it cannot measure asteroids.
	---------------------
	Remark
	Magnitudes equal to np.nan can be due to a number of causes. See photometry.aperture_photometry() for more information.
	---------------------
	Inputs
	* mag (3D Numpy array) = measured magnitudes. Has the form [[[m1_1_1, m1_1_2, ..., m1_1_P], [m1_2_1, m1_2_2, ..., m1_2_P], ..., [m1_N_1, M1_N_2, ..., M1_N_P]], ..., [[mi_1_1, mi_1_2, ..., mi_1_P], [mi_2_1, mi_2_2, ..., mi_2_P], ..., [mi_N_1, Mi_N_2, ..., Mi_N_P]]] where P=len(sizes) is the number of sets of photometry disks provided as input, N=len(coord) is the number of stars and i=len(Imagelist) the number of images. 
	* nbref (integer) = optional argument, default value is 20. Number of reference stars identified and then used to compute differential magnitudes. If there are not enough (less than nbref) stars without invalid measurements that can be used as reference stars then all the available reference stars are used.
	* plotting (boolean) = optional argument, default value is False. If True, a scatter plot showing mean magnitude vs magnitude RMS noise is displayed.
	---------------------
	Outputs
	* diff_mag (2D Numpy Array) = array of the form diff_mag[i,j], provides the best differential magnitudes on image i for object j.
	* diff_stats (list) = list of 3 Numpy arrays [diff_signal, diff_noise_global, diff_noise_local]. The arrays diff_signal[j], diff_noise_global[j], diff_noise_local[j] are providing for star j respectively: the average magnitude, the RMS magnitude noise and the RMS magnitude noise after filtering out slow variations.
	* quality_flag (list) = quality_flag[j] provides for each star j a flag indicating the quality of the measured differential magnitudes:
		0 --> measurements are fine
		1 --> measurements are slightly degraded but still fine
		2 --> measurements are a bit more degraded
		3 --> measurements have average quality
		4 --> measurements have poor quality
		-1 --> measurements have not been accepted because quality is much too low
	* which_disk (list) = which_disk[j] gives the index (starting at 0 so 0,1,2,...) of the set of photometry disks that was used for the best differential magnitude of object j.
	* ref_stars (2D Numpy Array) = array of the form ref_stars[j,n]. For each set j of photometry disks, provides the list of nbref best reference stars (or less than nbref if there are not enough good reference stars), ordered by increasing noise. The list of best reference stars is calculated based on 2 criteria: (1) lowest noise (2) lowest peak to peak fluctuations of differential magnitude over the images.
	---------------------
	"""
	print("")
	print("aspylib.photometry.compute_statistics():")
	
	mag2 = np.ma.masked_array(mag,np.isnan(mag))
	
	#-----------------------------------------------------------------------------------------------------------------------
	#		1- Computing statistics for raw magnitudes
	#-----------------------------------------------------------------------------------------------------------------------

	#--- filtering objects with >20% of wrong measurements (stars at edge, close to saturation, or wrong detections) ---
	test_star = np.sum(mag2.mask,axis=0)		#test_star[i,j]=N if the measurements on N images are equal to np.nan
	total = np.shape(mag2)[0]					#total = number of images
	
	for i in range(np.shape(mag2)[1]):		#loop on all measured objects
		for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
			if test_star[i,j] >= 0.2*total:
				mag2.mask[:,i,j]=True		
				mag2.data[:,i,j]=np.nan
				test_star[i,j] = total
		
	print("")
	for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
		print("disk:",j)
		print(str(np.sum(test_star[:,j] >= 0.2*total)) + "/" + str(len(test_star[:,j])) + " objects have more than 20% of wrong measurements, and have been removed")

	#--- computing statistics for remaining objects --- (commented as not used)
	#signal = np.mean(mag2,axis=0)
	#noise_global = np.std(mag2, ddof=0, axis=0)
	#for i in range(np.shape(mag2)[1]):		#loop on all measured objects
	#	for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
	#		if test_star[i,j] == total:
	#			signal[i,j]=np.nan
	#			noise_global[i,j]=np.nan

	noise_local = []
	for i in range(np.shape(mag2)[1]):		#loop on all measured objects
		noise_i = []
		for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
			ind = ~(mag2.mask[:,i,j])	#indexing unmasked measurements
			if np.sum(ind)==0:
				noise_i.append(np.nan)
			else:
				mag3 = mag2[:,i,j][ind]
				x = np.std(mag3[2:-2] - (mag3[0:-4]+mag3[1:-3]+mag3[2:-2]+mag3[3:-1]+mag3[4:])/5.,ddof=0, axis=0) / np.sqrt(1.-1./5.)
				noise_i.append(x)
		noise_local.append(noise_i)
		
	noise_local = np.array(noise_local)
	
	#test_star[i,j] = N if the measurements on N images are equal to np.nan
	#total = number of images
	#noise_local[i,j] = local noise with (i,j)=(nb_objects, nb_disks)
	
	#-----------------------------------------------------------------------------------------------------------------------
	#		2- Selecting reference stars
	#-----------------------------------------------------------------------------------------------------------------------

	#--- finding candidates for reference stars ---
	candid_ref = []
	noise_ref = []
	for i in range(np.shape(mag2)[1]):		#loop on all measured objects
		test = 0.
		for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
			test += test_star[i,j]				#test_star[i,j]=N if N measurements are equal to np.nan
		if test == 0.:							#test is the number of wrong measurements for object i
			candid_ref.append(i)
			noise_ref.append(noise_local[i,:])

	candid_ref = np.array(candid_ref)
	noise_ref = np.array(noise_ref)

	print("")
	print("nb of candidates for reference stars:", len(candid_ref))

	#--- sorting all reference star candidates ---
	ind = []
	for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
		ind.append(list(np.argsort(noise_ref[:,j])))
	indices = []
	for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
		indices.append(list(candid_ref[ind[j]]))

	#candid_ref = list of all reference stars that are candidates. They have all their measurements OK with all disks
	#--> what about if a disk or image is masked ? Reply = no images are masked, only certain objects for certain disks
	#indices --> indices[j] is the list of candidate reference stars, ordered by increasing noise, for the disk j

	#**************************************************************************
	#		Calculating magnitude offsets between different disk sizes
	#**************************************************************************
	
	#--- calculating a global rank for reference star candidates ---
	rank = []
	for i in range(len(candid_ref)):		#loop on reference star candidates
		drank = 0.
		for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
			drank += np.argsort(ind[j][:])[i]	#np.argsort(np.argsort(x)) replaces x=[0.34,0.12,0.234] with [2,0,1]: each value is replaced by its rank
		rank.append(drank)
	
	#--- calculating best global reference stars ---
	globind = np.argsort(rank)
	globindices = candid_ref[globind]
	
	#--- calculating offsets ---
	print("")
	print("magnitude offsets for each disk size")
	offset = []
	for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
		offset.append(np.mean(mag2[:,globindices[:20],j].flatten()))
	offset = np.array(offset)
	print("before averaging:", offset)
	offset -= np.mean(offset)
	print("after averaging:", offset)

	#rank = list, gives for each candidate reference star the sum of its rank for each disk. This allows to have a global sorting (across all disks) of reference stars
	#globindices = list of candidate reference stars, starting from the most preferred, across all disks
	#offset[j] gives an estimate of the magnitude offset linked to disk j
	
	#**************************************************************************
	#		Calculating best reference stars for each disk size
	#**************************************************************************
	
	#--- refining selection of reference stars, for each disk size ---
	for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
		indices[j] = list(indices[j][:2*nbref])
		
		for i in range(len(indices[j])-nbref):
			refmagtemp = np.mean(mag2[:,indices[j],j],axis=1)
			diffrefmagtemp = mag2[:,indices[j],j] - np.tile(refmagtemp,(len(indices[j]),1)).transpose()
			amplitude = np.max(diffrefmagtemp,axis=0) - np.min(diffrefmagtemp,axis=0)
			ind = list(np.argsort(amplitude))
			indices[j].remove(indices[j][ind[-1]])

		print("")
		print("disk size:",j)
		print("selected reference stars:",[i+1 for i in indices[j]])
	
	#indices[j,n] --> list of nbref best reference stars (or less than nbref if there are not enough good reference stars), ordered by increasing noise, for the disk j
	#it was initially calculated with the criterium of lowest noise. It is now refined with the additional criterium of lowest P2P fluctuation of differential magnitude
	
	#-----------------------------------------------------------------------------------------------------------------------
	#		3- Computing statistics for differential magnitudes
	#-----------------------------------------------------------------------------------------------------------------------

	#--- calculates average magnitude of nbref best reference stars (average on all reference stars) ---
	ref_mag = []
	for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
		ref_mag.append(np.mean(mag2[:,indices[j],j],axis=1))
	ref_mag = np.array(ref_mag)

	#ref_mag[j,i] = average magnitudes of the nbref reference stars for image i and disk j
	
	#--- corrects for the offsets due to difference in disk sizes and difference in set of selected reference stars ---
	mean_ref_mag = []
	for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
		mean_ref_mag.append(np.mean(ref_mag[j,:]))
	for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
		ref_mag[j,:] = ref_mag[j,:] - mean_ref_mag[j] + np.mean(mean_ref_mag) + offset[j]
	
	#(mean_ref_mag[j] - np.mean(mean_ref_mag)) = offsets due to difference in disk sizes and difference in set of selected reference stars
	#offset[j] = offsets due to difference in disk sizes, added here to the ref. stars so that it gets removed from the measured stars when computing the difference

	#--- differential magnitudes ---
	diff_mag = []
	for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
		#--- calculates differential magnitudes ---
		ref = np.tile(ref_mag[j,:],(np.shape(mag2)[1],1)).transpose()
		diff_mag.append(mag2[:,:,j] - ref)
	diff_mag = np.array(diff_mag)
	diff_mag = np.swapaxes(diff_mag,0,1)
	diff_mag = np.swapaxes(diff_mag,1,2)

	#diff_mag[i,n,j] = for image i, star n and disk j
	
	#--- computing statistics again ---
	diff_mag_masked = np.ma.masked_array(diff_mag,mask=mag2.mask)
	diff_signal = np.mean(diff_mag_masked,axis=0)
	diff_noise_global = np.std(diff_mag_masked, ddof=0, axis=0)
	for i in range(np.shape(mag2)[1]):		#loop on all measured objects
		for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
			if test_star[i,j] == total:			#double loop is required because if all values in diff_mag_masked = np.nan, then np.mean is equal to "masked" instead of np.nan
				diff_signal[i,j]=np.nan
				diff_noise_global[i,j]=np.nan
	
	diff_noise_local = []
	for i in range(np.shape(mag2)[1]):		#loop on all measured objects
		diff_noise_i = []
		for j in range(np.shape(mag2)[2]):		#loop on all disk sizes
			ind = ~(mag2.mask[:,i,j])	#indexing unmasked measurements : mag2 has the filter set to True when equal to np.nan
			if np.sum(ind)<=4:
				diff_noise_i.append(np.nan)
			else:
				diff_mag2 = diff_mag[:,i,j][ind]
				x = np.std(diff_mag2[2:-2] - (diff_mag2[0:-4]+diff_mag2[1:-3]+diff_mag2[2:-2]+diff_mag2[3:-1]+diff_mag2[4:])/5.,ddof=0, axis=0) / np.sqrt(1.-1./5.)
				diff_noise_i.append(x)
		diff_noise_local.append(diff_noise_i)

	diff_signal = np.array(diff_signal)
	diff_noise_global = np.array(diff_noise_global)
	diff_noise_local = np.array(diff_noise_local)

	#diff_signal[n,j], diff_noise_global[n,j], diff_noise_local[n,j] = for star n and disk j
	
	#**************************************************************************
	#		Keeping only best measurements
	#**************************************************************************
	
	best_ind = list(np.argmin(np.ma.masked_array(diff_noise_global,np.isnan(diff_noise_global)),axis=1))	#best_ind[i] is the index pointing to the best disk for image i. If all measurements are np.nan, np.argmin returns 0.
	i,j = np.ogrid[0:np.shape(diff_mag)[0],0:np.shape(diff_mag)[1]]
	diff_mag = diff_mag[i,j,best_ind]
		
	#--- re-computing statistics ---
	diff_signal = np.squeeze(diff_signal[j,best_ind])
	diff_noise_global = np.squeeze(diff_noise_global[j,best_ind])
	diff_noise_local = np.squeeze(diff_noise_local[j,best_ind])
	
	#diff_mag[i,j] for image i and star j
	#diff_signal[j], diff_noise_global[j], diff_noise_local[j] for star j
	
	#-----------------------------------------------------------------------------------------------------------------------
	#		4- Flagging quality of measurements
	#-----------------------------------------------------------------------------------------------------------------------
	
	quality_flag = []
	for i in range(np.shape(mag2)[1]):		#loop on all measured objects
		ind = ~(mag2.mask[:,i,best_ind[i]])
		if test_star[i,best_ind[i]]>=total-1:		#if all measurements are wrong, of if there is only one good measurement
			quality_flag.append(-1)
		else:
			c4 = (np.sum(np.abs(diff_mag[ind,i][1:]-diff_mag[ind,i][:-1])>0.50,axis=0) >  0)	#--> 2 successive values with more than 0.50 magnitude difference
			c3 = (np.sum(np.abs(diff_mag[ind,i][1:]-diff_mag[ind,i][:-1])>0.25,axis=0) >  0)
			c2 = (np.sum(np.abs(diff_mag[ind,i][1:]-diff_mag[ind,i][:-1])>0.10,axis=0) >  0)
			c1 = (np.sum(np.abs(diff_mag[ind,i][1:]-diff_mag[ind,i][:-1])>0.05,axis=0) >  0)
			
			quality_flag.append(c4*4 + c3*np.invert(c4)*3 + c2*np.invert(c3)*np.invert(c4)*2 + c1*np.invert(c2)*np.invert(c3)*np.invert(c4)*1)

	#0 --> measurement is fine
	#1 --> measurement is slightly degraded but still fine
	#2 --> measurement is a bit more degraded
	#3 --> measurement has average quality
	#4 --> measurement has poor quality
	#-1 --> measurement has not been accepted because quality is much too low
	
	#-----------------------------------------------------------------------------------------------------------------------
	#		5- Formatting results
	#-----------------------------------------------------------------------------------------------------------------------

	if plotting:
		plt.figure()
		plt.scatter(diff_signal,diff_noise_global)
		plt.grid()
		plt.xlabel('Instrumental magnitude')
		plt.ylabel('Magnitude noise')
		plt.show()

	diff_stats = [diff_signal, diff_noise_global, diff_noise_local]
	
	which_disk = np.array(best_ind)
	ref_stars = np.array(indices)
	
	return diff_mag, diff_stats, quality_flag, which_disk, ref_stars
