# -*- coding: iso_8859_1 -*-

import sys
from aspylib import conf
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.font_manager as fm

def filter_magnitudes(obj_index, names, sizes, mag_ok, rejdata):
	"""
	---------------------
	Purpose
	Given the measured magnitudes for a set of N targets (moving objects, variable stars or reference stars) over i images and with P sets of photometry disks, and the selection by the user of one particular target (moving object or variable star) this function detects and removes the measurements that have a poor quality.
	The function photometry.filter_magnitudes() must be called after photometry.check_refstars().
	After photometry.filter_magnitudes() has been applied, the complete data set does not contain any wrong measurement for the selected target (i.e. with magnitude np.nan). Additionally the user can control the quality of the data visually on a matplotlib plot and remove the points that he considers not good enough.
	---------------------
	Remark
	The following functions are meant to be used sequentially, with the outputs of a function being used as inputs for the next ones:
	1) photometry.check_refstars(names, mag) --> mag_ok, rejstar, rejdata
	2) photometry.filter_magnitudes(obj_index, names, sizes, mag_ok, rejdata) --> mag_i, rejdata_i, rejdisk_i
	3) photometry.best_magnitude(obj_index, names, sizes, mag_i, rejstar, rejdata_i, rejdisk_i, nb=10) --> ref_1, mag_1, sig_1
	4) photometry.write_best_magnitude(outputname, Imagelist, names, obj_index, sizes, dateobs, ref_1, mag_1, sig_1, rejdata_i, rejsizes=[], rejstar=[], nb=10, datation='start')
	OR
	photometry.write_cdrcdl_report(outputname, Imagelist, names, obj_index, dateobs, mag_i, ref_1, mag_1, sig_1, rejdata_i, format='DVR')
	---------------------
	Inputs
	* obj_index (integer) = index of the object that is being checked. If the list of measured objects is names=["moving_object_1", "moving_object_2", "vble_star_1", "ref_star_1", "ref_star_2"], obj_index=3 corresponds to "vble_star_1".
	* names (list) = list of strings, each string has the form "moving_object_i", "vble_star_i" or "ref_star_i" with i=1,2,... N and indicates the nature of each target.
	* sizes (list) = information about the photometry disks used to compute the magnitudes. List of the form [[center1, inner_radius1, outer_radius1], [center2, inner_radius2, outer_radius2], ..., [centerP, inner_radiusP, outer_radiusP]] where centeri, inner_radiusi and outer_radiusi are float numbers, which represent respectively 1) inneri = the radius of the disk within which the intensity of the star has been summed up 2) inner_radiusi and 3) outer_radiusi are the inner and outer radii defining a disk with a hole, that is centered on the star, within which the sky background was evaluated. The P elements correspond to P measurements of the star magnitude which were done in parallel.
	* mag_ok (3D Numpy masked array) = measured magnitudes. Has the form [[[m1_1_1, m1_1_2, ..., m1_1_P], [m1_2_1, m1_2_2, ..., m1_2_P], ..., [m1_N_1, M1_N_2, ..., M1_N_P]], ..., [[mi_1_1, mi_1_2, ..., mi_1_P], [mi_2_1, mi_2_2, ..., mi_2_P], ..., [mi_N_1, Mi_N_2, ..., Mi_N_P]]] where P=len(sizes) is the number of sets of photometry disks provided in "sizes", N=len(coord) is the number of stars and i=len(Imagelist) the number of images. The output "mag_ok" of photometry.check_refstars() must be used as the input "mag_ok" of photometry.filter_magnitudes().
	* rejdata (list) = indices of the images (starting at 0 = 0,1,2,3,...) that have already been identified as having bad quality, and therefore must be rejected. The output "rejdata" of photometry.check_refstars() must be used as the input "rejdata" of photometry.filter_magnitudes(). If all images are good, then rejdata=[] (empty list).
	---------------------
	Outputs
	* mag_i (3D Numpay masked array) = same as mag_ok, with additionally the data of poor quality for the particular target identified by its index "obj_index" being masked.
	* rejdata_i (list) = indices (starting at 0 = 0,1,2,3,...) of the images that have been identified as having bad quality for the particular target "obj_index". The list "rejdata_i" also contains the indices of the images of poor quality already identified before calling the function photometry.filter_magnitudes(), provided with the input parameter "rejdata". The data corresponding to "rejdata_i" is masked in "mag_i".
	* rejdisk_i (list) = indices (starting at 0 = 0,1,2,3,...) of the sets of photometry disks that correspond to bad quality measurements for the particular target "obj_index". The corresponding data is masked in "mag_i".
	---------------------
	"""
	print("")
	print("aspylib.photometry.filter_magnitudes():")
	
	nb_ref = len([i for i in names if (i[0]=='r')])
	nb_mov = len([i for i in names if (i[0]=='m')])
	nb_vbl = len([i for i in names if (i[0]=='v')])
	nb_obj = nb_mov + nb_vbl

	#------------------------------------------------------------------------------------	
	#		initialisations
	#------------------------------------------------------------------------------------	

	mag_i = mag_ok.copy()		#copy of masked array "mag_ok"
	
	rejdata_i = list(rejdata)
	rejdisk_i = []

	#------------------------------------------------------------------------------------	
	#		first check to remove wrong measurements affecting only the targets
	#		(not ref stars, for which wrong data have been already removed)
	#------------------------------------------------------------------------------------	

	print("")
	print('------ REMOVING WRONG MEASUREMENTS (IF ANY) -------------------------')
	print("")

	for i in range(len(mag_i)):
		if i not in rejdata:
			test_image = np.sum(np.isnan(mag_i[i,obj_index-1,:]).flatten())
			if test_image > 0:
				print("Image " + str(i+1) + " has wrong measurements, it is removed")
				rejdata_i.append(i)
				mag_i.mask[i,:,:] = True
			
	#------------------------------------------------------------------------------------	
	#		second check based on visual inspection
	#------------------------------------------------------------------------------------	
	
	fig = plt.figure()
	fig.canvas.set_window_title(names[obj_index-1]) 
	plt.axes([0.08,0.05,0.7,0.85])

	indices = ~(mag_i.mask[:,0,0])	#indexing unmasked images
	for j in range(len(sizes)):		
		mag_plot = mag_i[indices,obj_index-1,j] - np.mean(mag_i[:,nb_obj:,j], axis=1)[indices]
		ind = np.arange(1,np.sum(indices)+1)
		plt.plot(ind,mag_plot - np.mean(mag_plot), conf.col2[j%16], label=str(sizes[j]))
		
	plt.legend(prop=fm.FontProperties(size=11), bbox_to_anchor=(1.02, 1.), loc=2)	
	plt.title(names[obj_index-1])
	plt.grid()
	ax = plt.gca()
	ax.set_ylim(ax.get_ylim()[::-1])
	
	if (obj_index-1<nb_mov):
		print("")
		print('------ CHECKING DIFF MAGNITUDES FOR MOVING OBJECT',obj_index,'-----------------')
		print("")
		print('You have now the possibility to reject some data. For instance, data corresponding to the asteroid crossing a star is detected because it gives smaller magnitudes (larger fluxes) for larger disk sizes.')
	elif (obj_index-1>=nb_mov):
		print("")
		print('------ CHECKING DIFF MAGNITUDES FOR VARIABLE STAR',obj_index-nb_mov,'-----------------')
		print("")
		print('You have now the possibility to reject some data.')
	print('Do you want to do it (yes/no) ?')
	plt.show()

	if (sys.version[0]=="2"):
		ans = raw_input()
	elif (sys.version[0]=="3"):
		ans = input()

	if ans=='yes':
		#--------------------------------------------------------------------------------------------------
		if (len(sizes)>1):
			print("")
			print('Rejection of photometry disks...')
			print('if you want to reject the k-th specified disk, type k (for 1st disk write 1, 2nd disk write 2, etc) and then Enter')
			print('when finished, press Enter')

			while 1:
				try:
					fig = plt.figure()
					fig.canvas.set_window_title(names[obj_index-1]) 
					plt.axes([0.08,0.05,0.7,0.85])
					indices = ~(mag_i.mask[:,0,0])	#indexing unmasked images
					for j in range(len(sizes)):		
						mag_plot = mag_i[indices,obj_index-1,j] - np.mean(mag_i[:,nb_obj:,j], axis=1)[indices]
						ind = np.arange(1,np.sum(indices)+1)						
						plt.plot(ind,mag_plot - np.mean(mag_plot), conf.col2[j%16], label=str(sizes[j]))
					plt.legend(prop=fm.FontProperties(size=11), bbox_to_anchor=(1.02, 1.), loc=2)	
					plt.title(names[obj_index-1])
					plt.grid()
					ax = plt.gca()
					ax.set_ylim(ax.get_ylim()[::-1])
					print('disk to be rejected:')
					plt.show()
					
					if (sys.version[0]=="2"):
						val = int(raw_input())
					elif (sys.version[0]=="3"):
						val = int(input())

					rejdisk_i.append(val-1)
					mag_i.mask[:,:,val-1] = True
				except:
					break

		#masking data with rejected disks
		for ind in rejdisk_i:
			mag_i.mask[:,:,ind] = True
		
		#--------------------------------------------------------------------------------------------------
		print("")
		print('Rejection of photometric data...')
		print('how many intervals do you want to reject ?')

		while 1:
			try:
				fig = plt.figure()
				fig.canvas.set_window_title(names[obj_index-1]) 
				plt.axes([0.08,0.05,0.7,0.85])
				indices = ~(mag_i.mask[:,0,0])	#indexing unmasked images
				for j in range(len(sizes)):		
					mag_plot = mag_i[indices,obj_index-1,j] - np.mean(mag_i[:,nb_obj:,j], axis=1)[indices]
					ind = np.arange(1,np.sum(indices)+1)					
					plt.plot(ind,mag_plot - np.mean(mag_plot), conf.col2[j%16], label=str(sizes[j]))
				plt.legend(prop=fm.FontProperties(size=11), bbox_to_anchor=(1.02, 1.), loc=2)	
				plt.title(names[obj_index-1])
				plt.grid()
				ax = plt.gca()
				ax.set_ylim(ax.get_ylim()[::-1])
				plt.show()

				if (sys.version[0]=="2"):
					ans = int(raw_input())
				elif (sys.version[0]=="3"):
					ans = int(input())
				break
			except:
				print("please enter an integer !")
		for j in range(ans):
			print("")
			print('Interval',j+1)
			print('first rejected data:')
			while 1:
				try:
					fig = plt.figure()
					fig.canvas.set_window_title(names[obj_index-1]) 
					plt.axes([0.08,0.05,0.7,0.85])
					indices = ~(mag_i.mask[:,0,0])	#indexing unmasked images
					for j in range(len(sizes)):		
						mag_plot = mag_i[indices,obj_index-1,j] - np.mean(mag_i[:,nb_obj:,j], axis=1)[indices]
						ind = np.arange(1,np.sum(indices)+1)
						plt.plot(ind,mag_plot - np.mean(mag_plot), conf.col2[j%16], label=str(sizes[j]))
					plt.legend(prop=fm.FontProperties(size=11), bbox_to_anchor=(1.02, 1.), loc=2)	
					plt.title(names[obj_index-1])
					plt.grid()
					ax = plt.gca()
					ax.set_ylim(ax.get_ylim()[::-1])
					plt.show()
					if (sys.version[0]=="2"):
						mini = int(raw_input())-1
					elif (sys.version[0]=="3"):
						mini = int(input())-1
					break
				except:
					print("please enter an integer !")
			print('last rejected data:')
			while 1:
				try:
					fig = plt.figure()
					fig.canvas.set_window_title(names[obj_index-1]) 
					plt.axes([0.08,0.05,0.7,0.85])
					indices = ~(mag_i.mask[:,0,0])	#indexing unmasked images
					for j in range(len(sizes)):		
						mag_plot = mag_i[indices,obj_index-1,j] - np.mean(mag_i[:,nb_obj:,j], axis=1)[indices]
						ind = np.arange(1,np.sum(indices)+1)
						plt.plot(ind,mag_plot - np.mean(mag_plot), conf.col2[j%16], label=str(sizes[j]))
					plt.legend(prop=fm.FontProperties(size=11), bbox_to_anchor=(1.02, 1.), loc=2)	
					plt.title(names[obj_index-1])
					plt.grid()
					ax = plt.gca()
					ax.set_ylim(ax.get_ylim()[::-1])
					plt.show()
					if (sys.version[0]=="2"):
						maxi = int(raw_input())-1
					elif (sys.version[0]=="3"):
						maxi = int(input())-1
					break
				except:
					print("please enter an integer !")
			interval = range(min([mini,maxi]),max([mini, maxi])+1)
			rejdata_i = rejdata_i + list(np.where(indices)[0][interval])
			mag_i.mask[list(np.where(indices)[0][interval]),:,:] = True
		#---------------------------------------------------------------------

	return mag_i, rejdata_i, rejdisk_i
