# -*- coding: iso_8859_1 -*-

import numpy as np
import itertools as it 

def calc_noise(mag):
	"""
	---------------------
	Purpose
	Given a time series of magnitude values, this function evaluates the magnitude noise.
	It first filters out the slow variations with a window average. Then it computes the noise of the filtered data, and corrects it with the appropriate factor so that the obtained noise is representative of the original magnitude values (unfiltered).
	---------------------
	Inputs
	* mag (1D Numpy array) = magnitude values for which the noise must be computed. The length of the array should be len(mag) >= 6.
	---------------------
	Outputs
	* result (float) = estimated magnitude noise.
	---------------------
	"""
	if len(mag)<6:
		result = 'none'
	else:
		x = np.array(mag,dtype=np.float)
		y = x[2:-2] - (x[0:-4]+x[1:-3]+x[2:-2]+x[3:-1]+x[4:])/5.
		result = np.std(y,ddof=0) / np.sqrt(1.-1./5.)
		
	return result

def best_magnitude(obj_index, names, sizes, mag_i, rejstar, rejdata_i, rejdisk_i, nb=10):
	"""
	---------------------
	Purpose
	Given a set of measured magnitudes, this function allows to identify the best combination of reference stars and photometry disks.
	The function photometry.best_magnitude() must be called after photometry.check_refstars() and photometry.filter_magnitudes().
	The function computes the differential magnitudes (equal to the measured magnitude on target minus the average magnitude of the reference stars) for all possible combinations of reference stars and photometry disks. Then it computes the magnitude noise for every case. Finally the function returns the differential magnitudes, ordered by increasing magnitude noise (lowest noise first) for the "nb" best configurations.
	---------------------
	Remark
	The following functions are meant to be used sequentially, with the outputs of a function being used as inputs for the next ones:
	1) photometry.check_refstars(names, mag) --> mag_ok, rejstar, rejdata
	2) photometry.filter_magnitudes(obj_index, names, sizes, mag_ok, rejdata) --> mag_i, rejdata_i, rejdisk_i
	3) photometry.best_magnitude(obj_index, names, sizes, mag_i, rejstar, rejdata_i, rejdisk_i, nb=10) --> ref_1, mag_1, sig_1
	4) photometry.write_best_magnitude(outputname, Imagelist, names, obj_index, sizes, dateobs, ref_1, mag_1, sig_1, rejdata_i, rejsizes=[], rejstar=[], nb=10, datation='start')
	OR
	photometry.write_cdrcdl_report(outputname, Imagelist, names, obj_index, dateobs, mag_i, ref_1, mag_1, sig_1, rejdata_i, format='DVR')
	---------------------
	Inputs
	* obj_index (integer) = index of the object that is being checked. If the list of measured objects is names=["moving_object_1", "moving_object_2", "vble_star_1", "ref_star_1", "ref_star_2"], obj_index=3 corresponds to "vble_star_1".
	* names (list) = list of strings, each string has the form "moving_object_i", "vble_star_i" or "ref_star_i" with i=1,2,... N and indicates the nature of each target.
	* sizes (list) = information about the photometry disks used to compute the magnitudes. List of the form [[center1, inner_radius1, outer_radius1], [center2, inner_radius2, outer_radius2], ..., [centerP, inner_radiusP, outer_radiusP]] where centeri, inner_radiusi and outer_radiusi are float numbers, which represent respectively 1) inneri = the radius of the disk within which the intensity of the star has been summed up 2) inner_radiusi and 3) outer_radiusi are the inner and outer radii defining a disk with a hole, that is centered on the star, within which the sky background was evaluated. The P elements correspond to P measurements of the star magnitude which were done in parallel.
	* mag_i (3D Numpy masked array) = measured magnitudes. Has the form [[[m1_1_1, m1_1_2, ..., m1_1_P], [m1_2_1, m1_2_2, ..., m1_2_P], ..., [m1_N_1, M1_N_2, ..., M1_N_P]], ..., [[mi_1_1, mi_1_2, ..., mi_1_P], [mi_2_1, mi_2_2, ..., mi_2_P], ..., [mi_N_1, Mi_N_2, ..., Mi_N_P]]] where P=len(sizes) is the number of sets of photometry disks provided in "sizes", N=len(coord) is the number of stars and i=len(Imagelist) the number of images. The output "mag_i" of photometry.filter_magnitudes() must be used as the input "mag_i" of photometry.best_magnitude().
	* rejstar (list) = indices (starting at 0 = 0,1,2,3,...) of the rejected reference stars. Output of photometry.check_refstars().
	* rejdata_i (list) = indices (starting at 0 = 0,1,2,3,...) of the images that have been identified as having bad quality for the particular target "obj_index". The list "rejdata_i" also contains the indices of the images of poor quality for all targets. The data corresponding to "rejdata_i" is masked in "mag_i". Output of photometry.filter_magnitudes().
	* rejdisk_i (list) = indices (starting at 0 = 0,1,2,3,...) of the sets of photometry disks that correspond to bad quality measurements for the particular target "obj_index". The corresponding data is masked in "mag_i". Output of photometry.filter_magnitudes().
	* nb (integer) = optional argument, default value is 10. Number of configurations (a configuration is a set of reference stars, together with a set of photometry disks) returned by the function.
	---------------------
	Outputs
	* ref_1 (list) = list with an integer, and then a tuple of integers, e.g. [1, (1,3,4)]. The first integer represents the selected set of photometry disks (with indices 1,2,...,P for P disks). The tuple represents the selected reference stars (with indices 1,2,...,N for N reference stars).
	* mag_1 (2D Numpy array) = gives the "nb" best calculated differential magnitude that have the lowest noise. The differential magnitude is equal to the measured magnitude on the target minus the average magnitude of the selected reference stars. Array of the form [[m1_1,m1_2,...,m1_i], [m2_1,m2_2,...,m2_i], ..., [mk_1,mk_2,...,mk_i]] with i images and k=nb best magnitude calculations.
	* sig_1 (list) = list with the RMS magnitude noise values associated with the "nb" best magnitude calculations. 
	---------------------
	"""
	print("")
	print("aspylib.photometry.best_magnitude():")
	
	nb_obs = len([i for i in names if (i[0]=='m')or(i[0]=='v')])  
	nb_ref = len([i for i in names if (i[0]=='r')])  

	#values are the indices of the accepted reference stars
	values = np.array([i+1 for i in range(nb_ref) if i+nb_obs not in rejstar], dtype=np.int)
	
	print("")
	print("optimisation for ", names[obj_index-1])

	#initialisation
	mag_1 = []
	ref_1 = []
	sig_1 = []
	
	for size in range(len(sizes)):						#loop on apertures
		if size not in rejdisk_i:
			#print(sizes[size])
			for i in range(len(values)): 						#loop on nb of reference stars chosen
				for j in list(it.combinations(values,i+1)):		#loop on possible combinations of these reference stars
					#print(names[obj_index-1],sizes[size],"references stars",j)
					mag_2 = []
					
					for pict in range(len(mag_i)):						#loop on images
						if pict not in rejdata_i:
							mag_ref = 0.
							for k in range(len(j)):									#loop on reference stars
								mag_ref = mag_ref + mag_i[pict,nb_obs+j[k]-1,size] / len(j)			#mag_i[i,j,k] corresponds to image i star j disk k
							mag_2.append(mag_i[pict,obj_index-1,size] - mag_ref)

					mag_1.append(mag_2)
					ref_1.append([size, j])
					sig_1.append(calc_noise(mag_2))		#here if there is a single np.nan in the data then sig_2 is also np.nan (due to calc_noise)
														#this trick helps to prevent that there is any np.nan in the data after optimisation
	#pick up best results
	ord = np.argsort(sig_1)
	ref_1 = [ref_1[i] for i in ord[:nb]]
	mag_1 = np.array([mag_1[i] for i in ord[:nb]])
	sig_1 = [sig_1[i] for i in ord[:nb]]
	
	return ref_1, mag_1, sig_1
