# -*- coding: iso_8859_1 -*-

import os, datetime
import numpy as np
from aspylib import astro, conf

def write_best_magnitude(outputname, Imagelist, names, obj_index, sizes, dateobs, ref_1, mag_1, sig_1, rejdata_i, rejsizes=[], rejstar=[], nb=10, datation='start'):
	"""
	---------------------
	Purpose
	Saves the results of the function photometry.best_magnitude() in a text file. This text files contains the best differential magnitudes, with their corresponding choice of photometry disks and reference stars, and with the associated magnitude noise.
	---------------------
	Remark
	The following functions are meant to be used sequentially, with the outputs of a function being used as inputs for the next ones:
	1) photometry.check_refstars(names, mag) --> mag_ok, rejstar, rejdata
	2) photometry.filter_magnitudes(obj_index, names, sizes, mag_ok, rejdata) --> mag_i, rejdata_i, rejdisk_i
	3) photometry.best_magnitude(obj_index, names, sizes, mag_i, rejstar, rejdata_i, rejdisk_i, nb=10) --> ref_1, mag_1, sig_1
	4) photometry.write_best_magnitude(outputname, Imagelist, names, obj_index, sizes, dateobs, ref_1, mag_1, sig_1, rejdata_i, rejsizes=[], rejstar=[], nb=10, datation='start')
	OR
	photometry.write_cdrcdl_report(outputname, Imagelist, names, obj_index, dateobs, mag_i, ref_1, mag_1, sig_1, rejdata_i, format='DVR')
	---------------------
	Inputs
	* outputname (string) = complete name (including path) of a text file. First this filename will be modified by appending to it the name of the target corresponding to "obj_index" ; then the obtained filename will be used to save the information. If the file already exists it will be overwritten.
	* Imagelist (list) = series of FITS images over which the photometry has been computed. Has the form [string_1, string_2, ..., string_i]. Each list element string_k is the complete filename (including path) of one FITS image.
	* names (list) = list of strings, each string has the form "moving_object_i", "vble_star_i" or "ref_star_i" with i=1,2,... N and indicates the nature of each target.
	* obj_index (integer) = index of the object that is being checked. If the list of measured objects is names=["moving_object_1", "moving_object_2", "vble_star_1", "ref_star_1", "ref_star_2"], obj_index=3 corresponds to "vble_star_1".
	* sizes (list) = information about the photometry disks used to compute the magnitudes. List of the form [[center1, inner_radius1, outer_radius1], [center2, inner_radius2, outer_radius2], ..., [centerP, inner_radiusP, outer_radiusP]] where centeri, inner_radiusi and outer_radiusi are float numbers, which represent respectively 1) inneri = the radius of the disk within which the intensity of the star has been summed up 2) inner_radiusi and 3) outer_radiusi are the inner and outer radii defining a disk with a hole, that is centered on the star, within which the sky background was evaluated. The P elements correspond to P measurements of the star magnitude which were done in parallel.
	* dateobs (list) = contains the Julian dates of each image. It can be the Julian date at the time of start of exposure, or mid-exposure, i.e. start_exposure + exposure/2 (see also datation).
	* ref_1 (list) = list with an integer, and then a tuple of integers, e.g. [1, (1,3,4)]. The first integer represents the selected set of photometry disks (with indices 1,2,...,P for P disks). The tuple represents the selected reference stars (with indices 1,2,...,N for N reference stars). Output of photometry.best_magnitude().
	* mag_1 (2D Numpy array) = gives the "nb" best calculated differential magnitude that have the lowest noise. The differential magnitude is equal to the measured magnitude on the target minus the average magnitude of the selected reference stars. Array of the form [[m1_1,m1_2,...,m1_i], [m2_1,m2_2,...,m2_i], ..., [mk_1,mk_2,...,mk_i]] with i images and k=nb best magnitude calculations. Output of photometry.best_magnitude().
	* sig_1 (list) = list with the RMS magnitude noise values associated with the "nb" best magnitude calculations. Output of photometry.best_magnitude().
	* rejdata_i (list) = indices (starting at 0 = 0,1,2,3,...) of the images that have been identified as having bad quality for the particular target "obj_index", and therefore have be rejected and should not appear in the data stored in the file. If all images are good then rejdata_i=[] (empty list). Output of photometry.filter_magnitudes().
	* rejsizes (list) = optional argument, default value is []. Indices (starting at 0 = 0,1,2,3,...) of the sets of photometry disks that have been rejected for the particular target "obj_index", and therefore do not appear in the data stored in the file. If all sets of photometry disks are good, or alternatively if the user does not want to print the information in the file about which sets of photometry disks have been rejected, then rejsizes=[] (empty list).  The output "rejdisk_i" of photometry.filter_magnitudes() can be used here.
	* rejstar (list) = optional argument, default value is []. Indices (starting at 0 = 0,1,2,3,...) of the reference stars that have been rejected for the particular target "obj_index", and therefore do not appear in the data stored in the file. If all reference stars are good, or alternatively if the user does not want to print the information in the file about which reference stars have been rejected, then rejstar=[] (empty list). The output "rejstar" of photometry.check_refstars() can be used here.
	* nb (integer) = optional argument, default value is 10. Number of configurations (a configuration is a set of reference stars, together with a set of photometry disks) returned by the function.
	* datation (string) = optional argument. Defines whether if the date written in the text file is at the start (datation='start') or at the middle (datation='middle') of the exposure. Default value is 'start'. If the datation is at the start of exposure, the length of exposure, in seconds, is also written in the file.
	---------------------
	Output = no output
	---------------------
	"""
	
	nb_obj = len([i for i in names if (i[0]=='m')or(i[0]=='v')])  
	nb_ref = len([i for i in names if (i[0]=='r')])  
	nb_results = np.int(min([nb,len(sizes)*((2.**nb_ref)-1.)]))

	#---------- writing results to output file -------------	
	filename = outputname.split('.')[0] + '_' + names[obj_index-1] + '.' + outputname.split('.')[1]
	if os.path.exists(filename):
		os.remove(filename)

	f = open(filename, 'w')

	#--- file header ---
	f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write("Working directory:\n"+(os.path.split(Imagelist[0])[0])+"\n\n")

	f.write(str(len(Imagelist))+' images processed\n')
	f.write('first processed image = '+Imagelist[0].split(conf.sep)[-1]+'\n\n')

	if len(rejdata_i)>0:
		f.write('The following image(s) have been rejected:\n')
		for i in rejdata_i:
			f.write(Imagelist[i].split(conf.sep)[-1]+'\n')
		f.write("\n")			

	if len(rejsizes)>0:
		f.write('Disk sizes rejected:\n')
		for disk in rejsizes:
			f.write(str(sizes[disk])+'\n')
		f.write('\n')
	
	if len(rejstar)>0:
		f.write('Reference stars rejected:\n')
		for star in rejstar:
			f.write(str(names[star])+'\n')
		f.write('\n')
	
	if datation=='start':
		f.write('datation at start of exposure\n')
		h1 = astro.get_headervalues([Imagelist[0]],'EXPOSURE')[0]
		h2 = astro.get_headervalues([Imagelist[0]],'EXPTIME')[0]
		if h1!='':
			f.write('exposure time (sec):\t'+str(int(np.round(h1)))+'\n\n')
		elif h2!='':
			f.write('exposure time (sec):\t'+str(int(np.round(h2)))+'\n\n')
	elif datation=='middle':
		f.write('datation at middle of exposure\n')

	#--- measures magnitudes ---
	f.write("############### " + names[obj_index-1] + " ###############\n")
			
	#------ 1st line ------
	f.write("solutions")
	for i in range(nb_results):
		f.write("\t"+str(i+1))
	f.write("\n")
	
	#------ 2nd line ------
	f.write("apertures")
	for i in range(nb_results):
		f.write("\t"+str(sizes[ref_1[i][0]]))
	f.write("\n")

	#------ 3rd line ------
	f.write("reference stars")
	for i in range(nb_results):
		f.write("\t")
		f.write(str(ref_1[i][1][0]))	
		for j in range(len(ref_1[i][1])-1):		#loop on reference stars for selected method
			f.write('+'+str(ref_1[i][1][j+1]))
	f.write("\n")
	
	#------ 4th line ------
	f.write("magnitude noise")
	for i in range(nb_results):
		f.write("\t"+"%.6f"%sig_1[i])
	f.write("\n")

	#------ magnitudes ------
	f.write('Julian day')
	for i in range(nb_results):
		f.write('\tmag')
	f.write('\n')

	dateobs_rej = [dateobs[j] for j in range(len(Imagelist)) if j not in rejdata_i]
	for j in range(len(Imagelist)-len(rejdata_i)):
		f.write("%.6f"%dateobs_rej[j])
		for i in range(nb_results):
			f.write('\t'+"%.6f"%mag_1[i][j])
		f.write('\n')

	f.close()
	#-------------------------------------
