# -*- coding: iso_8859_1 -*-

import os
import numpy as np
from aspylib import astro

def write_cdrcdl_report(outputname, Imagelist, names, obj_index, dateobs, mag_i, ref_1, mag_1, sig_1, rejdata_i, format='DVR'):
	"""
	---------------------
	Purpose
	Saves the results of the function photometry.best_magnitude() in a file compatible with the CourbRot software:
		http://obswww.unige.ch/~behrend/redcouro/redcouro.html
		http://obswww.unige.ch/~behrend/page_fmt.html
	Two formats are supported:
		'DVR' = julian dates + measured magnitudes of the target + references stars
		'DVv' = julian dates + differential magnitude of the target (equal to the target measured magnitude minus the average measured magnitude of the reference stars) + RMS magnitude noise
	Julian dates are provided at the start of the exposure. No time corrections (i.e. heliocentric julian dates or time delay from light propagation) are done: the dates correspond to the date of the measurements.
	The file can be edited to write additional information, e.g. about the observer and/or the place of observation.
	---------------------
	Remark
	The following functions are meant to be used sequentially, with the outputs of a function being used as inputs for the next ones:
	1) mag_ok, rejstar, rejdata = photometry.check_refstars(names, mag)
	2) mag_i, rejdata_i, rejdisk_i = photometry.filter_magnitudes(obj_index, names, sizes, mag_ok, rejdata)
	3) ref_1, mag_1, sig_1 = photometry.best_magnitude(obj_index, names, sizes, mag_i, rejstar, rejdata_i, rejdisk_i, nb=10)
	4) photometry.write_best_magnitude(outputname, Imagelist, names, obj_index, sizes, dateobs, ref_1, mag_1, sig_1, rejdata_i, rejsizes=[], rejstar=[], nb=10, datation='start')
	OR
	photometry.write_cdrcdl_report(outputname, Imagelist, names, obj_index, dateobs, mag_i, ref_1, mag_1, sig_1, rejdata_i, format='DVR')
	---------------------
	Inputs
	* outputname (string) = complete name (including path) of a text file. First this filename will be modified by appending to it the name of the target corresponding to "obj_index" ; then the obtained filename will be used to save the information. If the file already exists it will be overwritten.
	* Imagelist (list) = series of FITS images over which the photometry has been computed. Has the form [string_1, string_2, ..., string_i]. Each list element string_k is the complete filename (including path) of one FITS image.
	* names (list) = list of strings, each string has the form "moving_object_i", "vble_star_i" or "ref_star_i" with i=1,2,... N and indicates the nature of each target.
	* obj_index (integer) = index of the object that is being checked. If the list of measured objects is names=["moving_object_1", "moving_object_2", "vble_star_1", "ref_star_1", "ref_star_2"], obj_index=3 corresponds to "vble_star_1".
	* dateobs (list) = contains the Julian dates of each image. The function assumes that the Julian date is at the time of start of exposure, and writes the CourbRot keyword "POS -1" in the file.
	* mag_i (3D Numpy masked array) = measured magnitudes. Has the form [[[m1_1_1, m1_1_2, ..., m1_1_P], [m1_2_1, m1_2_2, ..., m1_2_P], ..., [m1_N_1, M1_N_2, ..., M1_N_P]], ..., [[mi_1_1, mi_1_2, ..., mi_1_P], [mi_2_1, mi_2_2, ..., mi_2_P], ..., [mi_N_1, Mi_N_2, ..., Mi_N_P]]] where P=len(sizes) is the number of sets of photometry disks provided in "sizes", N=len(coord) is the number of stars and i=len(Imagelist) the number of images. "mag_i" will be used if the requested file format includes the measured magnitudes (rather than differential magnitudes). Output of photometry.filter_magnitudes().
	* ref_1 (list) = list with an integer, and then a tuple of integers, e.g. [1, (1,3,4)]. The first integer represents the selected set of photometry disks (with indices 1,2,...,P for P disks). The tuple represents the selected reference stars (with indices 1,2,...,N for N reference stars). "ref_1" will be used if the requested file format includes the measured magnitudes (rather than differential magnitudes). Output of photometry.best_magnitude().
	* mag_1 (2D Numpy array) = gives a list of the best calculated differential magnitude that have the lowest noise. The differential magnitude is equal to the measured magnitude on the target minus the average magnitude of the selected reference stars. Array of the form [[m1_1,m1_2,...,m1_i], [m2_1,m2_2,...,m2_i], ..., [mk_1,mk_2,...,mk_i]] with i images and k=nb best magnitude calculations. "mag_1" will be used if the requested file format includes differential magnitudes (rather than measured magnitudes). Output of photometry.best_magnitude().
	* sig_1 (list) = list with the RMS magnitude noise values associated with the "nb" best magnitude calculations. "sig_1" will be used if the requested file format includes differential magnitudes (rather than measured magnitudes). Output of photometry.best_magnitude().
	* rejdata_i (list) = indices (starting at 0 = 0,1,2,3,...) of the images that have been identified as having bad quality for the particular target "obj_index", and therefore have be rejected and should not appear in the data stored in the file. If all images are good then rejdata_i=[] (empty list). Output of photometry.check_refstars().
	* format (string) = optional argument, default value is 'DVR'. Can also be equal to 'DVv'. If format='DVR' then the file has 3 or more than 3 columns: 1 column for the Julian dates of the measurements, 1 column for the target measured magnitude, and additional columns for the measured magnitudes of each reference star. If format='DVv' then the file has exactly 3 columns: 1 column for the Julian dates of the measurements, 1 column for the target differential magnitude, and 1 column for the target RMS magnitude noise.
	---------------------
	Output = no output
	---------------------
	"""

	nb_obj = len([i for i in names if (i[0]=='m')or(i[0]=='v')])  
	
	filename = outputname.split('.')[0] + '_' + names[obj_index-1] + '.' + outputname.split('.')[1]

	#---------- writing results to output file -------------	
	if os.path.exists(filename):
		os.remove(filename)

	f = open(filename, 'w')

	if format=='DVR':
		#--- header FMT ---
		f.write('FMT DV')
		for i in range(len(ref_1[0][1])):	#loop on reference stars
			f.write('R')
		f.write('\n')

		#--- other mandatory headers ---
		f.write('NOM\nMES\n')

		#--- header POS ---
		h1 = astro.get_headervalues([Imagelist[0]],'EXPOSURE')[0]
		h2 = astro.get_headervalues([Imagelist[0]],'EXPTIME')[0]
		if h1!='':
			f.write('POS -1 ')
			f.write(str(int(np.round(h1))))
			f.write('\n')
		elif h2!='':
			f.write('POS -1 ')
			f.write(str(int(np.round(h2))))
			f.write('\n')
		
		#--- measures magnitudes ---
		for j in range(len(Imagelist)):
			if j not in rejdata_i:
				f.write(('%.6f' % dateobs[j]))
				f.write('\t'+('%.6f' % mag_i[j][obj_index-1][ref_1[0][0]]))		#index 0 for first (=best) result
				for i in range(len(ref_1[0][1])):	#loop on reference stars
					f.write('\t'+('%.6f' % mag_i[j][nb_obj+ref_1[0][1][i]-1][ref_1[0][0]]))		#index 0 for first (=best) result
				f.write("\n")

	elif format=='DVv':
		#--- header FMT ---
		f.write('FMT DVv\n')

		#--- other mandatory headers ---
		f.write('NOM\nMES\n')

		#--- header POS ---
		h1 = astro.get_headervalues([Imagelist[0]],'EXPOSURE')[0]
		h2 = astro.get_headervalues([Imagelist[0]],'EXPTIME')[0]
		if h1!='':
			f.write('POS -1 ')
			f.write(str(int(np.round(h1))))
			f.write('\n')
		elif h2!='':
			f.write('POS -1 ')
			f.write(str(int(np.round(h2))))
			f.write('\n')
		
		#--- measures magnitudes ---			
		dateobs_rej = [dateobs[j] for j in range(len(Imagelist)) if j not in rejdata_i]
		for j in range(len(Imagelist)-len(rejdata_i)):
			f.write(('%.6f' % dateobs_rej[j]))
			f.write('\t'+('%.6f' % mag_1[0][j]))
			f.write('\t'+('%.6f' % sig_1[0]) + '\n')

	f.close()
	#-------------------------------------

	