# -*- coding: iso_8859_1 -*-

import numpy as np
import os, datetime, pp, sys
from aspylib import astro, conf
import matplotlib.pyplot as plt

#about the use of pp with Python 3 --> see https://stackoverflow.com/questions/38908321/error-the-process-xxx-not-found-occurs-after-executing-parallel-python-code
	#PP works well with pypy (on Windows), however on shutdown, you get (harmless) failure messages from the invocation of 'TASKKILL' in pp.py line 823 (version 1.6.5) due to the child processes already being dead and absent at the time when the TASKKILL is called. This does not occur using normal CPython and I've not investigated why this is different between CPython and PyPy.
	#Nonetheless, since it is harmless, and the child processes are already gone, I'd like to suggest that the TASKKILL line is modified as follows to suppress the spurious error output. The TASKKILL line causing this error message is in the file pp.py. In version 1.6.5 of pp it is line 823. In version 1.6.4.7.1 of ppft it is in line 874 (near the end of the file). The file pp.py in my case was in the directory "Python36\Lib\site-packages\ppft-1.6.4.7.1-py3.6.egg\pp.py"
	#To suppress the error message, change the line from:
	#	os.popen('TASKKILL /PID '+str(worker.pid)+' /F')
	#to:
	#	os.popen('TASKKILL /PID '+str(worker.pid)+' /F 2>NUL')

def calculate_shifts(outputname, Imagelist, fit_area = 10, nb_stars = 12, parallel=True):
	"""
	---------------------
	Purpose
	For each image in a series, compute the translation [dx,dy] that must be applied to the 1st image to superimpose it to the ith image.
	It is assumed that the images are radiometrically calibrated. The function allows to control the quality of the computed shifts and repeat the computation on a subset if needed, and eventually flag the images for which the shift computation is not successful.
	The shift computation is based on detection of the brightest stars on each image, PSF fit to compute accurate centroid positions and comparing the star centroids between the images.
	The user can chose to use parallel computing thanks to the Python module pp (Parallel Python) to speed up the calculation.
	It is essential that the first image in the series has a good quality, as it will be used as a reference point for all other images.
	---------------------
	Inputs
	* outputname (string) = complete filename (including path) of the text file where the computed shifts and the flagged images will be stored.
	* Imagelist (list) = series of FITS images for which the shifts must be computed. Has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	* fit_area (integer, must be >0) = optional argument, default value is 10. Defines the +/- size of the square window used for star fitting (the square window has a size 2*fit_area+1).
	The window size must be large enough to include the stars for the PSF fit, but small enough not to include contributions from other stars.
	* nb_stars (integer) = optional argument, default value is 12. the number of stars used for the shift computation. The number must be large enough to remove all the ambiguities. Typical values are in the range 10-40.
	* parallel (boolean) = optional argument, default value is False. If True, parallel computing (Python module pp) is used ; otherwise computation is done in the standard way with one CPU.
	---------------------
	Outputs
	* Imagelist2 (list) = FITS images for which the shifts have been correctly computed. Has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	* shifts2 (2D Numpy array) = computed shifts. Has the shape (N,2) with N being the length of Imagelist2. Has the form [[x1,y1],[x2,y2],...,[xN,yN]].
	---------------------
	"""
	print("")
	print("aspylib.photometry.calculate_shifts():")

	ind = list(np.array(np.arange(1,len(Imagelist)+1), dtype=np.int))

	#------ loading image + initialising shifts ------
	shifts = [[0.,0.] for i in range(len(Imagelist))]
	image1 = astro.get_imagedata([Imagelist[0]], text_display=False)
	calculation_list = range(1,len(Imagelist))
	
	#------ initialising image area selection parameters ------
	excluded = []
	x0 = 0
	y0 = 0
	x1, y1 = np.shape(image1)
	
	#************************************************************************************
	while True:		
		print("")
		print('Image shift calculation can be perturbated by slowly moving objects, strongly saturated stars or artefacts. If the "star matching" method is used (default), diffuse objects may also be problematic.')
		print("")
		print('There are 3 possible approaches to calculate the shifts:')
		print('(1) perform star detections over the complete images')
		print('(2) exclude some image areas for the star detection')
		print('(3) detect only the stars in one subpart of the image')
		print("")
		print('What option do you want to use (1/2/3) ?')
		astro.display(image1, title="First image")
		while 1:
			try:
				if (sys.version[0]=="2"):
					ans = int(raw_input())
				elif (sys.version[0]=="3"):
					ans = int(input())
				break
			except:
				print("please enter an integer !")
		if ans==2:
			while 1:
				try:
					if (sys.version[0]=="2"):
						ans = int(raw_input('How many areas do you want to exclude ?'))
					elif (sys.version[0]=="3"):
						ans = int(input('How many areas do you want to exclude ?'))
					break
				except:
					print("please enter an integer !")
			for i in range(ans):
				print('Select 4 corners of area',i+1,'to be excluded')
				xy = astro.select(image1,nb=4,facecolor='red',marker='+')

				excluded.append([min([xy[0][0],xy[1][0],xy[2][0],xy[3][0]]), \
								max([xy[0][0],xy[1][0],xy[2][0],xy[3][0]]), \
								min([xy[0][1],xy[1][1],xy[2][1],xy[3][1]]), \
								max([xy[0][1],xy[1][1],xy[2][1],xy[3][1]])])
		elif ans==3:
			print('Select 4 corners of area to be used')
			xy = astro.select(image1,nb=4,facecolor='red',marker='+')

			x0 = min([xy[0][0],xy[1][0],xy[2][0],xy[3][0]])
			x1 = max([xy[0][0],xy[1][0],xy[2][0],xy[3][0]])
			y0 = min([xy[0][1],xy[1][1],xy[2][1],xy[3][1]])
			y1 = max([xy[0][1],xy[1][1],xy[2][1],xy[3][1]])

		#------------------------------------------------------------------------------------
		print("")
		stars1 = astro.find_stars_accurate(image1[x0:x1,y0:y1], nb_stars, fit_area, excluded=excluded, text_display = False)
		
		print('shifts of image :',Imagelist[0].split(conf.sep)[-1])
		print("dx:",0.)
		print("dy:",0.)
		#shifts[0] = [0.,0.]		#not needed (shifts is initialised with [0.,0.] for all images at beginning of the function)
		
		if parallel:

			def parallel_imageshift(i, Imagelist, x0, x1, y0, y1, nb_stars, fit_area, excluded, stars1):
				print("")
				print('calculating shifts of image :',Imagelist[i].split(aspylib.conf.sep)[-1])
				image = aspylib.astro.get_imagedata([Imagelist[i]], text_display=False)
				stars = aspylib.astro.find_stars_accurate(image[x0:x1,y0:y1], nb_stars, fit_area, excluded=excluded, text_display = False)
				xs, ys = aspylib.astro.find_shift_stars_matching(stars1, stars, text_display = False)
				print("dx:",xs)
				print("dy:",ys)
				return [xs,ys]

			# tuple of all parallel python servers to connect with
			job_server = pp.Server()
			print("Starting pp with", job_server.get_ncpus(), "workers")
			
			jobs = []
			for i in calculation_list:	
				jobs.append(job_server.submit(parallel_imageshift,(i, Imagelist, x0, x1, y0, y1, nb_stars, fit_area, excluded, stars1,), (), ("aspylib.astro", "aspylib.conf",)))
				
			for i in range(len(jobs)):
				shifts[calculation_list[i]] = jobs[i]()
			
			job_server.print_stats()

		else:		
			#Here code that can be executed if the parallel computation is not used:
			for i in calculation_list:	
				print('calculating shifts of image :',Imagelist[i].split(conf.sep)[-1])
				image = astro.get_imagedata([Imagelist[i]], text_display=False)
				stars = astro.find_stars_accurate(image[x0:x1,y0:y1], nb_stars, fit_area, excluded=excluded, text_display = False)
				xs, ys = astro.find_shift_stars_matching(stars1, stars, text_display = False)
				print("dx:",xs)
				print("dy:",ys)
				shifts[i] = [xs,ys]

		#------------------------------------------------------------------------------------
		fig = plt.figure()
		fig.canvas.set_window_title("Image shifts") 
		plt.plot(ind,np.array(shifts)[:,0], 'b')
		plt.plot(ind,np.array(shifts)[:,1], 'g')
		plt.grid()
		plt.title('Image shifts: Vertical (blue), Horizontal (green)')
		plt.xlabel('Image number')
		plt.ylabel('Image shifts (pixels)')

		print("")
		print('You must now choose one of the 4 possibilities:')
		print('(1) you accept all the calculated shifts')
		print('(2) you accept the calculated shifts but you want to remove a few points')
		print('(3) you accept only part of the calculated shifts, you want to recalculate the shifts for a few points')
		print('(4) you are not happy with the results, you want to restart all the calculation')
		print("")
		print('What is your decision (1/2/3/4) ?')
		plt.show()	
		while 1:
			try:
				if (sys.version[0]=="2"):
					choice = int(raw_input())
				elif (sys.version[0]=="3"):
					choice = int(input())
				break
			except:
				print("please enter an integer !")

		if (choice == 1)or(choice == 2):	#the shifts are accepted, but maybe a few points will be removed
			break
		elif choice == 3:					#the shifts are partly accepted, and a few points will be re-calculated	
			calculation_list = []
			print("")
			print('You now need to specify for which images you want to recalculate the shifts.')
			while 1:
				try:
					if (sys.version[0]=="2"):
						im = int(raw_input("Please enter the numbers of the images for shift re-calculation:"))
					elif (sys.version[0]=="3"):
						im = int(input("Please enter the numbers of the images for shift re-calculation:"))
					calculation_list.append(im-1)
				except:
					break
			calculation_list = list(set(calculation_list))
			calculation_list.sort()
		else:								#the shifts are rejected and the calculation is re-started from the beginning
			shifts = [[0.,0.] for i in range(len(Imagelist))]
			calculation_list = range(1,len(Imagelist))
		
		
	rej = []
	if (choice==1):		#all calculated shifts are accepted
		pass
	elif (choice==2):	#the calculated shifts are accepted but a few points will be removed
		print("")
		print("")
		print('--- IMAGES WITH INCORRECT SHIFT CALCULATION -------------------------')
		print("")
		print('You have now the possibility to reject some images for which the shift was not correctly calculated.')
		while 1:
			try:
				if (sys.version[0]=="2"):
					im = int(raw_input("Please enter the numbers of the images to be removed:"))
				elif (sys.version[0]=="3"):
					im = int(input("Please enter the numbers of the images to be removed:"))
				rej.append(im-1)
			except:
				break
		rej = list(set(rej))
		rej.sort()

	#------------------------------------------------------------------------------------
	if os.path.exists(outputname):
		os.remove(outputname)

	f = open(outputname, 'w')
	f.write('date:\t'+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write("Working directory:\n"+os.path.split(Imagelist[0])[0]+"\n\n")
	f.write("-------------------------------------------------\n")
	f.write("      User inputs\n")
	f.write("-------------------------------------------------\n")
	f.write("Images rejected by user:\t")
	if len(rej)==0:
		f.write('none\n')
	else:
		for i in range(len(rej)-1):
			f.write(str(rej[i]+1)+',')
		f.write(str(rej[-1]+1)+"\n")
	f.write("\n")
	
	f.write("-------------------------------------------------\n")
	f.write("      Images shifts are\n")
	f.write("-------------------------------------------------\n")
	f.write('Image\tImage status\tImage name\tX shift (vertical)\tY shift (horizontal)\n')
	for i,im in enumerate(Imagelist):
		f.write(str(i+1)+'\t')
		if i not in rej:
			f.write("ok\t")
		else:
			f.write("rejected\t")
		f.write(im.split(conf.sep)[-1]+'\t'+"%.4f"%shifts[i][0]+'\t'+"%.4f"%shifts[i][1]+'\n')
	f.close()
	
	#------------------------------------------------------------------------------------
	Imagelist2 = [Imagelist[i] for i in range(len(Imagelist)) if i not in rej]
	shifts2 = np.array([shifts[i] for i in range(len(shifts)) if i not in rej])

	return Imagelist2, shifts2
