# -*- coding: iso_8859_1 -*-

import numpy as np
from aspylib import astro

def load_shifts(filename):
	"""
	---------------------
	Purpose
	Loads the file that contains the image shifts that was created during execution of the function photometry.calculate_shifts().
	The function will ignore the images that have been rejected.
	---------------------
	Inputs
	* filename (string) = complete filename (including path) of the text file where the computed shifts have been stored.
	---------------------
	Outputs
	* imagelist (list) = series of FITS images for which the shift computation was succesfull (i.e. not flagged). Has the form [string_1, string_2, ..., string_N]. Each list element string_i is the complete filename (including path) of one FITS image.
	* shifts (2D Numpy Array) = shifts of the accepted (i.e. not flagged) images. Has the form [[x1,y1], [x2,y2], ..., [xN,yN]] for N images.
	---------------------
	"""

	folder = astro.load_data(filename, "Working directory", "\n")[0][0]
	data = astro.load_data(filename, "Image\tImage status", "\n", colstart=1)
	
	imagelist = [folder+"\\"+im for im in list(data[data[:,0]=="ok",1])]
	shifts = np.array(data[data[:,0]=="ok", 2:], dtype=np.float)
	
	return imagelist, shifts

def load_imagestats(filename):
	"""
	---------------------
	Purpose
	Reads the file that contains the information about image statistics (min, max, sky background) that was created during execution of the function photometry.check_pictures_quality().
	The function will ignore the data for the images that have been rejected.
	---------------------
	Inputs
	* filename (string) = complete filename (including path) of the text file where the information about radiometric quality has been stored.
	---------------------
	Outputs
	* imagelist (list) = series of FITS images with acceptable radiometric quality (i.e. not flagged). Has the form [string_1, string_2, ..., string_N]. Each list element string_i is the complete filename (including path) of one FITS image.
	* globminmax (2D Numpy Array) = reports the global minima and maxima of each of the N images. Has the form [[min1, max1], [min2, max2], ..., [minN, maxN]].
	* minmax (2D Numpy Array) = reports the local minima and maxima of each of the N images, in the small rectangular area that was selected by the user during the radiometric test. Has the form [[min1, max1], [min2, max2], ..., [minN, maxN]].
	* bgnd (2D Numpy Array) = reports the non-uniformity of the sky background over the image. When testing the image, 5 areas are defined automatically (at the 4 corners and at the image center). The median values are computed at these 5 positions within squares of 100*100 pixels, and are reported after subtracting the average of the 5 median values. So [a-mean(a,b,c,d,e), b-mean(a,b,c,d,e), ..., e-mean(a,b,c,d,e)] are reported. For an image with uniform background, the 5 values must be very close to each other (no outlier). For an image with a non-uniform background, 1 or more values will be different from the others. The returned variable "bgnd" has the form [[bgnd1_1, bgnd2_1, bgnd3_1, bgnd4_1, bgnd5_1], [bgnd1_2, bgnd2_2, bgnd3_2, bgnd4_2, bgnd5_2], ..., [bgnd1_N, bgnd2_N, bgnd3_N, bgnd4_N, bgnd5_N]] for N images.
	* varbgnd (2D Numpy Array) = reports the variations in time of the non-uniformity of the sky background over the image. From the values of bgnd computed as explained above, varbgnd is computed by subtracting from bgnd its average value over the image series. Therefore, varbgnd = [[bgnd1_1-mean(bgnd1), bgnd2_1-mean(bgnd2), bgnd3_1-mean(bgnd3), bgnd4_1-mean(bgnd4), bgnd5_1-mean(bgnd5)], ...]. The average is removed in the function photometry.check_pictures_quality(), BEFORE identifying the images that must be removed. The average is subtracted again in the current function photometry.load_imagestats(), since it is now computed only with the valid images.
	---------------------
	"""

	folder = astro.load_data(filename, "Working directory", "\n")[0][0]
	data = astro.load_data(filename, "Image\tImage status", "\n", colstart=1)

	imagelist = [folder+"\\"+im for im in list(data[data[:,0]=="ok",1])]
	stats = np.array(data[data[:,0]=="ok", 2:], dtype=np.float)
	
	globminmax = stats[:,:2]
	minmax = stats[:,2:4]
	bgnd = stats[:,4:9]
	varbgnd = stats[:,9:14]
	varbgnd -= np.mean(varbgnd,axis=0)
	
	return imagelist, globminmax, minmax, bgnd, varbgnd
