# -*- coding: iso_8859_1 -*-

import os, datetime
import numpy as np
from aspylib import astro, conf

def write_magnitudes(outputname, Imagelist, names, coord, dateobs, sizes, mag, datation='start'):
	"""
	---------------------
	Purpose
	Writes a text file that saves all the information about the photometry computed with the functions photometry.make_photometry and photometry.aperture_photometry.
	- date of file writing
	- folder + image names
	- stars coordinates on first image
	- computed magnitudes + corresponding Julian dates
	---------------------
	Inputs
	* outputname (string) = complete name (including path) of the text file where the information will be written. If the file already exists it will be overwritten.
	* Imagelist (list) = series of FITS images over which the photometry has been computed. Has the form [string_1, string_2, ..., string_i]. Each list element string_k is the complete filename (including path) of one FITS image.
	* names (list) = contains the names of all stars for which photometry has been computed. Has the form [string_1, string_2, ..., string_N] for N stars. Typical names are ["star_1", "star_2", ..., "star_N"].
	* coord (3D Numpy array) = array of the form [[[x11,y11], [x12,y12], ..., [x1N,y1N]],...,[[xi1,yi1], [xi2,yi2], ..., [xiN,yiN]]]. It gives the [x,y] coordinates for the N stars that have been photometrically measured over the i images. The variable coord[i] contains the coordinates of all stars on the ith image with filename Imagelist[i].
	* dateobs (list) = contains the Julian dates of each image. It can be the Julian date at the time of start of exposure, or mid-exposure, i.e. start_exposure + exposure/2 (see also datation).
	* sizes (list) = list of the form [[center1, inner_radius1, outer_radius1], [center2, inner_radius2, outer_radius2], ..., [centerP, inner_radiusP, outer_radiusP]] where centeri, inner_radiusi and outer_radiusi are float numbers, which represent respectively 1) inneri = the radius of the disk within which the intensity of the star was summed up 2) inner_radiusi and 3) outer_radiusi are the inner and outer radii defining a disk with a hole, that is centered on the star, within which the sky background was evaluated. The P elements correspond to P measurements of the star magnitude which were done in parallel.
	* mag (2D Numpy array) = measured magnitudes to be written to the file. Has the form [[m1_1, m1_2, ..., m1_P], [m2_1, m2_2, ..., m2_P], ..., [mN_1, MN_2, ..., MN_P]]. P=len(sizes) is the number of sets of photometry disks provided as input and N=len(coord) is the number of stars. 
	* datation (string) = optional argument. Defines whether if the date written in the text file is at the start (datation='start') or at the middle (datation='middle') of the exposure. Default value is 'start'. If the datation is at the start of exposure, the length of exposure in seconds is also written in the file.
	---------------------
	Outputs (no outputs)
	---------------------
	"""

	if os.path.exists(outputname):
		os.remove(outputname)

	f = open(outputname, 'w')

	#--- file header ---
	f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write("Working directory:\n"+(os.path.split(Imagelist[0])[0])+"\n\n")

	if datation=='start':
		f.write('datation at start of exposure\n')
		h1 = astro.get_headervalues([Imagelist[0]],'EXPOSURE')[0]
		h2 = astro.get_headervalues([Imagelist[0]],'EXPTIME')[0]
		if h1!='':
			f.write('exposure time (sec):\t'+str(int(np.round(h1)))+'\n\n')
		elif h2!='':
			f.write('exposure time (sec):\t'+str(int(np.round(h2)))+'\n\n')
	elif datation=='middle':
		f.write('datation at middle of exposure\n')
		
	#--- stars X,Y coordinates ---
	f.write('objects coordinates on first frame\n')
	f.write('object')
	for j in range(len(coord[0])):
		f.write('\t'+names[j])
	f.write('\n')
	f.write('X position (vertical)')
	for j in range(len(coord[0])):
		f.write('\t'+"%.4f"%coord[0][j][0])
	f.write('\n')
	f.write('Y position (horizontal)')
	for j in range(len(coord[0])):
		f.write('\t'+"%.4f"%coord[0][j][1])
	f.write('\n\n')

	#--- measured magnitudes ---
	for k in range(len(sizes)):
		f.write('magnitudes from apertures: '+str(sizes[k][0])+' - '+str(sizes[k][1])+' - '+str(sizes[k][2])+'\n')

		#magnitudes
		f.write('Image\tImage name\tJulian day')
		for j in range(len(coord[0])):
			f.write('\t'+names[j])
		f.write('\n')
		for i,im in enumerate(Imagelist):
			f.write(str(i+1)+'\t')
			f.write(im.split(conf.sep)[-1]+'\t')
			f.write(('%.6f' % dateobs[i]))
			for j in range(len(coord[i])):
				f.write('\t'+'%.6f' % mag[i,j,k])
			f.write('\n')
		f.write('\n')

	f.close()
	#-------------------------------------
	
def write_magnitudes_and_SNR(outputname, Imagelist, names, coord, dateobs, sizes, mag, SNR, datation='start'):
	"""
	---------------------
	Purpose
	Writes a text file that saves all the information about the photometry computed with the functions photometry.make_photometrystats and photometry.aperture_photometrystats.
	- date of file writing
	- folder + image names
	- stars coordinates on first image
	- computed magnitudes and SNR + corresponding Julian dates
	---------------------
	Inputs
	* outputname (string) = complete name (including path) of the text file where the information will be written. If the file already exists it will be overwritten.
	* Imagelist (list) = series of FITS images over which the photometry has been computed. Has the form [string_1, string_2, ..., string_i]. Each list element string_k is the complete filename (including path) of one FITS image.
	* names (list) = contains the names of all stars for which photometry has been computed. Has the form [string_1, string_2, ..., string_N] for N stars. Typical names are ["star_1", "star_2", ..., "star_N"].
	* coord (3D Numpy array) = array of the form [[[x11,y11], [x12,y12], ..., [x1N,y1N]],...,[[xi1,yi1], [xi2,yi2], ..., [xiN,yiN]]]. It gives the [x,y] coordinates for the N stars to be photometrically measured over the i images. The variable coord[i] contains the coordinates of all stars on the ith image with filename Imagelist[i].
	* dateobs (list) = contains the Julian dates of each image. It can be the Julian date at the time of start of exposure, or mid-exposure, i.e. start_exposure + exposure/2 (see also datation).
	* sizes (list) = list of the form [[center1, inner_radius1, outer_radius1], [center2, inner_radius2, outer_radius2], ..., [centerP, inner_radiusP, outer_radiusP]] where centeri, inner_radiusi and outer_radiusi are float numbers, which represent respectively 1) inneri = the radius of the disk within which the intensity of the star is summed up 2) inner_radiusi and 3) outer_radiusi are the inner and outer radii defining a disk with a hole, that is centered on the star, within which the sky background is evaluated. The P elements correspond to P measurements of the star magnitude which are done in parallel with different sets of parameters.
	* mag (2D Numpy array) = measured magnitudes to be written to the file. Has the form [[m1_1, m1_2, ..., m1_P], [m2_1, m2_2, ..., m2_P], ..., [mN_1, MN_2, ..., MN_P]]. P=len(sizes) is the number of sets of photometry disks provided as input and N=len(coord) is the number of stars. 
	* SNR (2D Numpy array) = same as mag, containes the SNR values instead of the magnitudes.
	* datation (string) = optional argument. Defines whether if the date written in the text file is at the start (datation='start') or at the middle (datation='middle') of the exposure. Default value is 'start'. If the datation is at the start of exposure, the length of exposure, in seconds, is also written in the file.
	---------------------
	Outputs (no outputs)
	---------------------
	"""
	
	if os.path.exists(outputname):
		os.remove(outputname)

	f = open(outputname, 'w')

	#--- file header ---
	f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write("Working directory:\n"+(os.path.split(Imagelist[0])[0])+"\n\n")

	if datation=='start':
		f.write('datation at start of exposure\n')
		h1 = astro.get_headervalues([Imagelist[0]],'EXPOSURE')[0]
		h2 = astro.get_headervalues([Imagelist[0]],'EXPTIME')[0]
		if h1!='':
			f.write('exposure time (sec):\t'+str(int(np.round(h1)))+'\n\n')
		elif h2!='':
			f.write('exposure time (sec):\t'+str(int(np.round(h2)))+'\n\n')
	elif datation=='middle':
		f.write('datation at middle of exposure\n')
		
	#--- stars X,Y coordinates ---
	f.write('objects coordinates on first frame\n')
	f.write('object')
	for j in range(len(coord[0])):
		f.write('\t'+names[j])
	f.write('\n')
	f.write('X position (vertical)')
	for j in range(len(coord[0])):
		f.write('\t'+"%.4f"%coord[0][j][0])
	f.write('\n')
	f.write('Y position (horizontal)')
	for j in range(len(coord[0])):
		f.write('\t'+"%.4f"%coord[0][j][1])
	f.write('\n\n')

	#--- measured magnitudes ---
	for k in range(len(sizes)):
		f.write('magnitudes from apertures: '+str(sizes[k][0])+' - '+str(sizes[k][1])+' - '+str(sizes[k][2])+'\n')

		#magnitudes
		f.write('Image\tImage name\tJulian day')
		for j in range(len(coord[0])):
			f.write('\t'+names[j])
		f.write('\n')
		for i,im in enumerate(Imagelist):
			f.write(str(i+1)+'\t')
			f.write(im.split(conf.sep)[-1]+'\t')
			f.write(('%.6f' % dateobs[i]))
			for j in range(len(coord[i])):
				f.write('\t'+'%.6f' % mag[i,j,k])
			f.write('\n')
		f.write('\n')

	#--- measured SNRs ---
	for k in range(len(sizes)):
		f.write('SNRs from apertures: '+str(sizes[k][0])+' - '+str(sizes[k][1])+' - '+str(sizes[k][2])+'\n')

		#magnitudes
		f.write('Image\tImage name\tJulian day')
		for j in range(len(coord[0])):
			f.write('\t'+names[j])
		f.write('\n')
		for i,im in enumerate(Imagelist):
			f.write(str(i+1)+'\t')
			f.write(im.split(conf.sep)[-1]+'\t')
			f.write(('%.6f' % dateobs[i]))
			for j in range(len(coord[i])):
				f.write('\t'+'%.2f' % SNR[i,j,k])
			f.write('\n')
		f.write('\n')

	f.close()
	#-------------------------------------